#!/store/bin/perl5
##!/usr/bin/perl
##
##          File: dialup.pl
##   Description: Dialup service for Radius ISDN/Analog
##      Language: Perl/5
##       Version: 1.00
##        Author: Sveinar Rasmussen (sveinar@rasmussen.org)
##     Copyright: IT-avdelingen, University of Troms, Norway.
##          Date: October 1998
##
##            Thanks to Brge Brunes for all the help!
##---------------------------------------------------------------
use GD;

$CONFIG_FILE                 = "dialup.conf";
$CONFIG_REV                  = 1;

$USE_MAILX                   = 1;  # 0=no, 1=yes

$MAILXPATH		     = "/usr/bin/mailx";
$MAILPATH		     = "/bin/mail";
$MAILSUB_OPTION              = "-s";

#-----------------------------------------------
$VERSION                     = "  v1.0  ";
#-----------------------------------------------
# debug functions
#===============================================
sub dd { 
    if ($LOGGINGENABLED!=1) {
	print "[info]: @_ \n"; 
    }
}
sub dd_elem {
  local ($di) = @_;
  dd "dat[$di]{NAS_IP}     =      $dat[$di]{'NAS_IP'}";
  dd "dat[$di]{start_time} =      $dat[$di]{'start_time'}";
  dd "dat[$di]{stop_time}  =      $dat[$di]{'stop_time'}";
  dd "dat[$di]{port}       =      $dat[$di]{'port'}";
  dd "dat[$di]{type}       =      $dat[$di]{'type'}";
  dd "dat[$di]{user}       =      $dat[$di]{'user'}";
  dd "dat[$di]{session_id} =      $dat[$di]{'session_id'}";
  dd "dat[$di]{in_bytes}   =      $dat[$di]{'in_bytes'}";
  dd "dat[$di]{out_bytes}  =      $dat[$di]{'out_bytes'}";
  dd "dat[$di]{time_used}  =      $dat[$di]{'time_used'}";
  dd "dat[$di]{phone}      =      $dat[$di]{'phone'}";
  dd "-------------------------";
}

#-----------------------------------------------
# Error handler
#===============================================
sub err {
  local ($errno, $val) = @_;
  if ($errno==1) { dd "Couldn't open ${CONFIG_FILE}. Aborted."; }
  elsif ($errno==2) { dd "Parse error in config file at line $val"; }
  elsif ($errno==3) { dd "Unexpected setting in config file at line $val"; }
  elsif ($errno==4) { dd "Couldn't open logfile $val. Aborted."; }
  elsif ($errno==5) { dd "Internal error $val. Aborted."; }
  elsif ($errno==6) { dd "Try another data file, please. Aborted."; }
  elsif ($errno==7) { dd "Missing logentry for beginning of week. Aborted."; }
  elsif ($errno==8) { dd "Inconsistent NAS-IP for week report in $val. Aborted."; }
  elsif ($errno==9) { dd "Inconsistent number of modems for week report in $val. Aborted."; }
  elsif ($errno==10) { dd "Old configuration file $val. Aborted."; }
  elsif ($errno==11) { dd "File not found: $val. Aborted."; }
  elsif ($errno==12) { dd "Error writing file: $val. Aborted."; }

  exit($errno);
}

%monList =  ('Jan', "01", 'Feb', "02", 'Mar', "03", 'Apr', "04", 'May', "05", 
	     'Jun', "06", 'Jul', "07", 'Aug', "08", 'Sep', "09", 'Oct', "10", 
	     'Nov', "11", 'Dec', "12");
%RmonList = ('01', "Jan", '02', "Feb", '03', "Mar", '04', "Apr", '05', "May",
	     '06', "Jun", '07', "Jul", '08', "Aug", '09', "Sep", '10', "Oct",
	     '11', "Nov", '12', "Dec");
%RmonListEx = ('01', "January", '02', "February", '03', "March", 
	       '04', "April", '05', "May", '06', "June", '07', "July",
	       '08', "August", '09', "September", '10', "October",
	       '11', "November", '12', "December");
%daysList = ('0', "Mon", '1', "Tue", '2', "Wed", '3', "Thu",
	     '4', "Fri", '5', "Sat", '6', "Sun" ); 
%ndays =    ('01', 31, '02', 28, '03', 31, '04', 30, '05', 31, 
	     '06', 30, '07', 31, '08', 31, '09', 30, '10', 31, 
	     '11', 30, '12', 31);

#-----------------------------------------------
# Parse config file
#===============================================
sub checkBase {
  local ($val) = @_;

  if (substr($val, 1, 8) =~ /PathBase/) {
    if (defined $PathBase) {
      $PathBase =~ /.*(.)$/;
      if ($1 =~ /\//) { chop $PathBase; }
      local $tmp = substr($val, 9);
      $val = $PathBase . $tmp;
    } else { $val = substr($val, 9); }
  }
  return $val;
}

sub parse_config_file {
  
  open CFGFILE, "${CONFIG_FILE}" || err(1,0);
  local $var = "";
  local $val = "";
  local $tmp = "";
  local $linenum = 1;
  $line = "";
  $line = <CFGFILE>;
  while ($line ne "") {
    local @curline = split /\ |\t\n/,$line;
    local $checkchar = substr $curline[0], 0, 1;
    if ( ($checkchar ne "#") && ($checkchar ne "\n") && 
         ($checkchar ne "" )) {
      if ( $line =~ "\"" ) { 
	$line =~ /(\w+)\s*(\#*)\s*\"(\#*.+)\"(.*|\n)/; 
	$var = "$1 ";
	$val = $3;
	$tmp = $2;
      } else { 
	$line =~ /(\w+)\s*(\#*\d+)(.*|\n)/; 
	$var = "$1 ";
	$val = $2;
	$tmp = "";
      }

      if (($val =~ "#") || ($tmp =~ "#") || 
	  ($val eq "")) { err(2, "$linenum:\n\t$line"); }
      if ($var =~    /LogfileBase\ /) {
	$val = checkBase($val);
	$LOGFILE1 = "$val"; }
      elsif ($var =~ /LogfileContinuous\ /) {
	$val = checkBase($val);
	$LOGFILECONT = "$val"; }
      elsif ($var =~ /DataOutDir\ /) {
	$val = checkBase($val);
	$DATADIR = "$val"; }
      elsif ($var =~ /HtmlOutDir\ /) {
	$val = checkBase($val);
	$HTMLDIR = "$val"; }
      elsif ($var =~ /SilentMode\ /) {
	$LOGGINGENABLED = "$val"; }
      elsif ($var =~ /WebRoot\ /) {
	$WEBROOT = "$val"; }
      elsif ($var =~ /GfxOutDir\ /) {

        if (substr($val, 1, 8) =~ /PathBase/) {
           $GFXDIRR = substr($val, 9);
        } else { $GFXDIRR = "NotDefined"; }

	$val = checkBase($val);
	$GFXDIR = "$val"; }
      elsif ($var =~ /TmpDir\ /) {
	$val = checkBase($val);
	$TMPDIR = "$val"; }
      elsif ($var =~ /SimultaneousAnalogThreshold\ /) {
	$SIMULTANEOUS_THRESHOLD = "$val"; }
      elsif ($var =~ /TotalPorts\ /) {
	$TOTAL_PORTS = "$val"; }
      elsif ($var =~ /TotalAnalogPorts\ /) {
	$TOTAL_ANALOG = "$val"; }
      elsif ($var =~ /GraphSignature\ /) {
	$GRAPH_SIGNATURE = "$val"; }
      elsif ($var =~ /GraphInitialTile\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_TILE = "$val"; }
      elsif ($var =~ /GraphInitialHourly\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_HOURLY = "$val"; }
      elsif ($var =~ /PathBase\ /) {
	$PathBase = "$val"; }
      elsif ($var =~ /GraphInitialAnalog\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_ANALOG = "$val"; }
      elsif ($var =~ /GraphInitialYear\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_YEAR = "$val"; }
      elsif ($var =~ /WarningsEnabled\ /) {
	$WARNINGSENABLED = "$val"; }
      elsif ($var =~ /WarningsEmailAdr\ /) {
	$WARNINGSEMAIL = "$val"; }
      elsif ($var =~ /HtmlInitialHeading\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_HEADING = "$val"; }
      elsif ($var =~ /HtmlInitialHeadWeek\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_HEADING_WEEK = "$val"; }
      elsif ($var =~ /HtmlInitialHeadContinuous\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_HEADING_CONT = "$val"; }
      elsif ($var =~ /HtmlInitialHeadMonth\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_HEADING_MONTH = "$val"; }
      elsif ($var =~ /HtmlInitialTailWeek\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_TAIL_WEEK = "$val"; }
      elsif ($var =~ /HtmlIndexDayHeading\ /) {
	$val = checkBase($val);
	$HTML_INDEX_DAY_HEAD = "$val"; }
      elsif ($var =~ /HtmlIndexMonthHeading\ /) {
	$val = checkBase($val);
	$HTML_INDEX_MONTH_HEAD = "$val"; }
      elsif ($var =~ /HtmlIndexWeekHeading\ /) {
	$val = checkBase($val);
	$HTML_INDEX_WEEK_HEAD = "$val"; }
      elsif ($var =~ /HtmlInitialTailMonth\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_TAIL_MONTH = "$val"; }
      elsif ($var =~ /HtmlInitialTailContinuous\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_TAIL_CONT = "$val"; }
      elsif ($var =~ /GraphInitialWeekly\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_WEEKLY = "$val"; }
      elsif ($var =~ /GraphInitialMonthly\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_MONTHLY = "$val"; }
      elsif ($var =~ /GraphInitialWeeklyAnalog\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_WEEKLY_ANALOG = "$val"; }
      elsif ($var =~ /2YearReportStart\ /) {
	$YEAR_REPORT_START = "$val"; }
      elsif ($var =~ /GraphInitialMonthlyAnalog\ /) {
	$val = checkBase($val);
	$GRAPH_INITIAL_MONTHLY_ANALOG = "$val"; }
      elsif ($var =~ /ConfigReqRevision\ /) {
	local $tmp = $val;
	if ($tmp < $CONFIG_REV) { err(10,"(need rev. $CONFIG_REV)"); }
      } elsif ($var =~ /HtmlInitialTail\ /) {
	$val = checkBase($val);
	$HTML_INITIAL_TAIL = "$val"; }
      elsif ($var eq "") { }
      else { err(3, "$linenum:\n\t$line"); }
    }
    $linenum++;
    $line = <CFGFILE> || last;
  }

  close CFGFILE;
}

#-----------------------------------------------
# Show parameters usage
#===============================================
sub show_param_usage {
  print STDERR "
Usage: $0 [OPTION]...\n
   -logfile  LOGFILENAME       use specified logfile base.
   -function FUNCTION          specify function:
                                   daily weekly monthly
                                   2year continuous
   -startReport YEAR-MM-DD     specify start for generating
                               report. Applies to weekly (DD
                               may be any day of the week. 
                               Default: last week) and
                               monthly (DD ignored. Default:
                               last month).
   -import DIR -prefix NAME    import and process all logs
                               in directory DIR with NAME
                               as filename prefix.

Report bugs to sveinar\@rasmussen.org
Copyright (c) 1998 IT-Avdelingen, UiTo.\n\n";
  exit(-1);
}

#-----------------------------------------------
# Parse arguments
#===============================================
sub parse_args {
  local (@arg) = @_;

  $LOGFILE_SET = 0;
  $function = "daily";
  local $curArg = shift @arg;
  while ($curArg ne "") {
    if ($curArg =~ /-logfile/) {
      $curArg = shift @arg;
      if ($curArg eq "") { show_param_usage(); }
      $LOGFILE1 = $curArg;
      $LOGFILE_SET = 1;
    } elsif ($curArg =~ /-function/) {
      $curArg = shift @arg;
      if ( ($curArg eq "") || 
	   ( ($curArg ne "daily") && ($curArg ne "weekly") 
	     && ($curArg ne "monthly") &&
	     ($curArg ne "2year") && ($curArg ne "continuous") ) )
	{ show_param_usage(); }
      $function = $curArg;
    } elsif ($curArg =~ /-import/) {
      $IMPORT_FOLDER = shift @arg;
      $curArg = shift @arg;
      if ($curArg eq "-prefix") {
	$IMPORT_PREFIX = shift @arg;
	$function = "import";
      } else { show_param_usage(); }
    } elsif ($curArg =~ /-startReport/) {

      $curArg = shift @arg;
      if ($curArg eq "") { show_param_usage(); }
      local @arr = split /-/,"$curArg-";

      local $tst = substr ($arr[1],0,1);
      local $valueA = int $arr[1];
      local $valueB = int $arr[2];

      if (($valueA < 10) && ($tst ne "0")) { $arr[1] = "0$valueA"; }
      if (($valueB < 10) && ($tst ne "0")) { $arr[2] = "0$valueB"; }

      $overrideStartReport = "$arr[0]-$arr[1]-$arr[2]";
    } else { show_param_usage(); }
    $curArg = shift @arg || last;
  }
}

#-----------------------------------------------
# open_data_file
#===============================================
sub open_data_file {
  local @str = split /\ /, "$date ";
  if ($str[2] < 10) { $str[2] = "0$str[2]"; }
  $rdate = "$str[3]-$monList{$str[1]}-$str[2]";

  open DATAFILE, ">${DATADIR}$rdate.data" or
      err(12, "${DATADIR}$rdate.data");

  datafile_addtag("date");
  datafile_put("$date");
  datafile_endtag();
}

#-----------------------------------------------
# open_html_file
#===============================================
sub open_html_file {
  local ($filenameDst, $src) = @_;

  open HTMLFILE, ">$filenameDst" or err(12, "$filenameDst");
  open HTMLSTART, "$src" or err(11, "$src");
  $line = <HTMLSTART>;
  while ($line ne "") { 
    print HTMLFILE $line;
    $line = <HTMLSTART> || last;
  }
  close HTMLSTART;
}

#-----------------------------------------------
# put data file start tag
#===============================================
sub datafile_addtag {
  local ($tag) = @_;
  print DATAFILE "<$tag>";
}

#-----------------------------------------------
# put data file end tag
#===============================================
sub datafile_endtag {
  print DATAFILE "</>\n";
}

#-----------------------------------------------
# put data file data
#===============================================
sub datafile_put {
  local ($data) = @_;
  print DATAFILE "$data";
}

#-----------------------------------------------
# put html file data
#===============================================
sub htmlfile_put {
  local ($data) = @_;
  print HTMLFILE "$data\n";
}

#----------------------------------------------
# close_data_file
#===============================================
sub close_data_file {
  close DATAFILE;
}

#----------------------------------------------
# close_html_file
#===============================================
sub close_html_file {
  local ($tail) = @_;
  open HTMLTAIL, "$tail" or err(11,"$tail");
  $line = <HTMLTAIL>;
  while ($line ne "") { 
    print HTMLFILE $line;
    $line = <HTMLTAIL> || last;
  }
  close HTMLTAIL;
  close HTMLFILE;
}

#----------------------------------------------
# load_file , in: FILE HANDLE
#===============================================
sub load_file {
  local ($fname, $state) = @_;
  if ($state =~ /init/) {
    open LOGFILE, $fname or err(4, $fname);
  } else {
    open LOGFILE, $fname or goto dontload;
  }

  $line = <LOGFILE>;
  while ( $line ne "" ) {
    if ($state =~ /init/) {

      chop $line;
      $line =~ /(\w+)\ (\w+)\ +(\d+)\ \d+:\d+:\d+\ (\d+).*/;
      $date = "$1 $2 $3 $4";
      $state = "";
    }
    if ($state =~ /extra/) {
      local @tmp = split /\ /,$line;
      $olddate = $date;
      $date = "$tmp[0] $tmp[1] $tmp[2] $tmp[4]";
      $state = "eks";
    }

    $cmpline = $line;
    if ($cmpline =~ /(\w+)\s+(\w+)\s+(\d+)\s+\d+:\d+:\d+\s+(\d+)/) {
      $cmpline = "$1 $2 $3 $4";
    } else { $cmpline = ""; }
    if ( ($cmpline ne "") && ($cmpline =~ /$date/) ) { 

      $line =~ /(\w+)\s+(\w+)\s+(\d+)\s+(\d+:\d+:\d+)\s+(\d+)/;
      $phrase[0] = $1;
      $phrase[1] = $2;
      $phrase[2] = $3;
      $phrase[3] = $4;
      $phrase[4] = $5;

      $dat[$dent]{'stop_time'} = $phrase[3];
      local $line = <LOGFILE>;
      while ( $line ne "\n" ) {
	local @phrase = split /=\ /, $line; chop ($phrase[1]);
	if ($phrase[0] =~ /NAS-IP-Address /) {
	  for ($i=0; $i != $number_NAS_IPs; $i++) { 
	    if ($NAS_IPs[$i] =~ $phrase[1]) { last; }
	  };
	  if ($i==$number_NAS_IPs) {
	    $NAS_IPs[$i] = $phrase[1];
	    $number_NAS_IPs++;
	  }
	  $dat[$dent]{'NAS_IP'} = $i; 
	}
	elsif ($phrase[0] =~ /NAS-Port /) {
	  $dat[$dent]{'port'}       = $phrase[1]; }
	elsif ($phrase[0] =~ /NAS-Port-Type /) {
	  $dat[$dent]{'type'}       = $phrase[1]; }
	elsif ($phrase[0] =~ /User-Name /) {
	  $dat[$dent]{'user'}       = $phrase[1]; }
	elsif ($phrase[0] =~ /Acct-Session-Id /) {
	  $dat[$dent]{'session_id'} = $phrase[1]; }
	elsif ($phrase[0] =~ /Acct-Input-Octets /) {
	  $dat[$dent]{'in_bytes'}   = $phrase[1]; }
	elsif ($phrase[0] =~ /Acct-Output-Octets /) {
	  $dat[$dent]{'out_bytes'}  = $phrase[1]; }
	elsif ($phrase[0] =~ /Acct-Session-Time /) {
	  $dat[$dent]{'time_used'}  = $phrase[1]; }
	elsif ($phrase[0] =~ /Calling-Station-Id /) {
	  $dat[$dent]{'phone'}  = $phrase[1]; }
	
	$line = <LOGFILE>;
      }
      
      if (defined $dat[$dent]{'out_bytes'}) {
	$dat[$dent]{'start_time'} = find_start ($dat[$dent]{'stop_time'}, 
						$dat[$dent]{'time_used'} );
	local @startT = split /:/,$dat[$dent]{'start_time'};
	local @endT = split /:/,$dat[$dent]{'stop_time'};
	if ($startT[0] > $endT[0]) { # off scale? adjust either way
	  if ($state =~ /eks/) {
	    $dat[$dent]{'stop_time'} = "23:59:59";
	    $ratio = ($dat[$dent]{'time_used'} - 3600*$endT[0] -
			      60*$endT[1] - $endT[2]) /
			      $dat[$dent]{'time_used'};
	  } else {
	    $dat[$dent]{'start_time'} = "00:00:00";
	    $ratio = (3600*$endT[0] + 60*$endT[1] + $endT[2]) /
	      $dat[$dent]{'time_used'};
	  }
	  $dat[$dent]{'time_used'} = int ($dat[$dent]{'time_used'}*$ratio);
	  $dat[$dent]{'in_bytes'} = int ($dat[$dent]{'in_bytes'}*$ratio);
	  $dat[$dent]{'out_bytes'} = int ($dat[$dent]{'out_bytes'}*$ratio);
	  if ($state =~ /eks/) { $dent++; }
	}
	if (!($state =~ /eks/)) { $dent++; } # ignore if start event
      }
    }
    $line = <LOGFILE> || last;
  }

  if ($state =~ /eks/) { $date = $olddate; }
  close LOGFILE;
 dontload:
}

#----------------------------------------------
# convert_size -> MB
#===============================================
sub convert_size {
  local ($number) = @_;
  local $str;
  local $tmp_pos;
  
  $str = $number / 1048576;
  $tmp_pos = (rindex $str, ".") + 3;
  $str = substr $str, 0, $tmp_pos;
  $str  .= " MB"; 

  return $str;
}

#----------------------------------------------
# convert_size MB -> GB
#===============================================
sub convert_size_GB {
  local ($number) = @_;
  local $str;
  local $tmp_pos;
  
  $str = $number / 1024;
  $tmp_pos = (rindex $str, ".") + 3;
  $str = substr $str, 0, $tmp_pos;
  $str .= " GB";

  return $str;
}

#----------------------------------------------
# find_start
#===============================================
sub find_start {
  local ($end, $dur) = @_;

  ($eH, $eM, $eS) = split /:/, $end;
  local $durH = int ($dur / 3600);
  local $durM = int (($dur - $durH*3600) / 60);
  local $durS = $dur - $durH*3600-$durM*60;
  local $sS = $eS-$durS; if ($sS < 0) { $durM++; $sS += 60; }
  local $sM = $eM-$durM; if ($sM < 0) { $durH++; $sM += 60; }
  local $sH = $eH-$durH; if ($sH < 0) { $sH = 0; }
  
  return "$sH:$sM:$sS";
}

#----------------------------------------------
# diff_time
#===============================================
sub diff_time {
  local ($time1, $time2) = @_;
  
  ($T1H, $T1M, $T1S) = split /:/, $time1;
  ($T2H, $T2M, $T2S) = split /:/, $time2;
  return abs(($T2H*3600+$T2M*60+$T2S) - ($T1H*3600+$T1M*60+$T1S));
}

#----------------------------------------------
# get_seconds (time), return seconds
#===============================================
sub get_seconds {
  local ($end) = @_;
  
  ($eH, $eM, $eS) = split /:/, $end;
  return $eH*3600+$eM*60+$eS;
}

#----------------------------------------------
# add_port (hour, port number)
#===============================================
sub add_port {
  local ($hour, $port, $list) = @_;
  if (($list < 0) || ($hour < 0)) { 
    dd "Warning: Inconsistency check failed (list,hour=$list, $hour)"
  } else {
    if (!($hour_port[$list][$hour] =~ / $port /)) {
      $hour_port[$list][$hour] .= " $port "
    }
  }
}

#----------------------------------------------
# Inspect hour for total busy time
#===============================================
sub inspect_hour {
  local ($hour, $type, $current_NAS, $max_nodes) = @_;

# PSEUDO - Foerste innlysende trege versjon....
#
# A = finn aktive porter av TYPE klokka $hour:00:00;
# T = 00
# U = 00
# D = 00
# while D!=59 and U!=59
#   D = neste nedkopling etter T || 59;
#   U = neste oppkopling etter T || 59;
# 
#   if (U <= D) then
#         A++;
#         T = U;
#   if (D <= U) then
#         if A=total then
#              S += D-T
#         A--;
#         T = D;
# 

  local $A = 0;
  local $U = 0;
  local $D = 0;
  local $addTime = 0;

  # finn aktive porter klokka $hour:00:00
  local $lookTime = $hour*3600;
  local $T = $lookTime;
  local $endTime = $lookTime + 3599;
  local $starTime = 0;
  local $stopTime = 0;
  for (local $i=0; $i < $dent; $i++) {
    if (($current_NAS==$dat[$i]{'NAS_IP'}) &&
	($dat[$i]{'type'} =~ /$type/) ) {
      $dat[$i]{'start_time'} =~ /(\d+):(\d+):(\d+)/;
      $starTime = $1*3600 + $2*60 + $3;
      $dat[$i]{'stop_time'} =~ /(\d+):(\d+):(\d+)/;
      $stopTime = $1*3600 + $2*60 + $3;
      if (($lookTime<$stopTime) && ($lookTime>=$starTime)) { $A++; }
    }
  }

  while ( ($U < $endTime) && ($D < $endTime) ) {
    $U = $endTime;
    $D = $endTime;
    for (local $i=0; $i < $dent; $i++) {
      if (($current_NAS==$dat[$i]{'NAS_IP'}) &&
	  ($dat[$i]{'type'} =~ /$type/) ) {
	$dat[$i]{'start_time'} =~ /(\d+):(\d+):(\d+)/;
	$starTime = $1*3600 + $2*60 + $3;
	$dat[$i]{'stop_time'} =~ /(\d+):(\d+):(\d+)/;
	$stopTime = $1*3600 + $2*60 + $3;

	if (($starTime < $endTime) &&
	    ($starTime > $T) ) {
	  $U = $starTime < $U ? $starTime : $U;
	}
	if (($stopTime < $endTime) &&
	    ($stopTime > $T) ) {
	  $D  = $stopTime < $D ? $stopTime : $D;
	}
      }
    }
    
    if ($U <= $D) {
      $A++;
      $T = $U;
    }
    if ($D <= $U) {
      if ($A==$max_nodes) { $addTime += $D - $T; }
      $A--;
      $T = $D;
    }
  }

  return $addTime;
}

#-----------------------------------------------
# main_per_nas
#===============================================
sub main_per_nas {
  local ($current_NAS) = @_;

  dd "Analyzing NAP-IP $NAS_IPs[$current_NAS]. Please wait...";

  for (local $k=0; $k < 24; $k++) { # clear array
    $hour_port[0][$k] = ""; 
    $hour_port[1][$k] = "";
    $MODEM_usage[$k+1] = 0;   # modem 1-->24
  }
  for (local $k=-1; $k < $usr_id; ) { # clear user structure
    $k++;
    $user[$k]{user} = "";
    $user[$k]{last_time} = "";
    $user[$k]{total_ISDN_time} = "";
    $user[$k]{total_MODEM_time} = "";
  }
  $usr_id = 0;

  local $tot_in_bytes = 0;
  local $tot_out_bytes = 0;
  local $tot_bundled = 0;
  local $tot_ISDN_time = 0;
  local $tot_MODEM_time = 0;
  local $MODEM_dialups = 0;
  local $MODEM_suspects = "";
  local $ISDN_suspects = "";
  for (local $i=0; $i < $dent; $i++) {
    if ($current_NAS==$dat[$i]{'NAS_IP'}) {
      # for graph

      local @startT = split /:/,$dat[$i]{'start_time'};
      local @endT = split /:/,$dat[$i]{'stop_time'};
      local $arrayPos = 0;
      if ($dat[$i]{'type'} =~ /ISDN-Sync/) { $arrayPos = 1; }
      for (local $j=$startT[0]; $j <= $endT[0]; $j++) {
	add_port ($j, $dat[$i]{'port'}, $arrayPos);
      }
      
      # count different users and measure time usage
      # subtract for bundled connections with ISDN
      local $k=0;
      while (($k ne $usr_id) && 
	     ($user[$k]{user} !~ /$dat[$i]{'user'}/)) { $k++; }
      if ($k eq $usr_id) { # new entry
	$user[$k]{user} = $dat[$i]{'user'};
	$user[$k]{last_time} = $i;

	if ($dat[$i]{'type'} =~ /ISDN-Sync/) {
	  $user[$k]{total_ISDN_time} = $dat[$i]{'time_used'};
	  $user[$k]{total_MODEM_time} = 0;
	} else {
	  $user[$k]{total_ISDN_time} = 0;
	  $user[$k]{total_MODEM_time} = $dat[$i]{'time_used'};
	}
	$usr_id++;
      } else { # old entry, update
	if ($dat[$i]{'type'} =~ /ISDN-Sync/) {
	  $user[$k]{total_ISDN_time} += $dat[$i]{'time_used'};
	} else { $user[$k]{total_MODEM_time} += $dat[$i]{'time_used'}; }

	if ($dat[$i]{'stop_time'} !~ /23:59:59/) {
	  local $cur_h =get_seconds($dat[$i]{'stop_time'});
	  local $last_h=get_seconds($dat[$user[$k]{last_time}]{'stop_time'});

	  if ( ($cur_h eq $last_h) || 
	       ((($cur_h-$dat[$i]{'time_used'}) < $last_h) &&
		($cur_h-$dat[$i]{'time_used'} ne 0)) ) {

	    if ($dat[$i]{'type'} =~ /ISDN-Sync/) {
	      if ( $dat[$i]{'time_used'} > 
		   $dat[$user[$k]{last_time}]{'time_used'} ) {
		$user[$k]{total_ISDN_time} -= 
		  $dat[$user[$k]{last_time}]{'time_used'};
		$tot_bundled += $dat[$user[$k]{last_time}]{'time_used'};
	      } else { 
		$user[$k]{total_ISDN_time} -= $dat[$i]{'time_used'}; 
		$tot_bundled += $dat[$i]{'time_used'};
	      }
	      $ISDN_suspects .= "$i-$user[$k]{last_time} ";
	    } else {
	      if (($dat[$i]{'type'} =~ /Async/) && 
		  ($dat[$user[$k]{last_time}]{'type'} =~ /Async/) &&
		  ( ($cur_h-$dat[$i]{'time_used'}) >
		    ($last_h-$dat[$user[$k]{last_time}]{'time_used'}) ) ) {
		$MODEM_suspects .= "$i-$user[$k]{last_time} ";
	      }
	    }
	  }
	}
	$user[$k]{last_time} = $i;
      }
      
      # count octets in/out and hours used of the server
      if ($dat[$i]{'type'} =~ /ISDN-Sync/) {
	$tot_ISDN_time += $dat[$i]{'time_used'};
      } else { 
	$MODEM_dialups++;
	$tot_MODEM_time += $dat[$i]{'time_used'}; 

	# gather MODEM usage, 1-24
	$MODEM_usage[$dat[$i]{'port'}] += $dat[$i]{'time_used'};

      }
      $tot_in_bytes += $dat[$i]{'in_bytes'};
      $tot_out_bytes += $dat[$i]{'out_bytes'};
      
    }
  }
  
  # now, count users and their time usage to find average
  local $total_avg_ISDN_time = 0;
  local $total_avg_MODEM_time = 0;
  local $ISDN_user_cnt = 0;
  local $MODEM_user_cnt = 0;
  local $BOTH_user_cnt = 0;
  for (local $k=0; $k < $usr_id; $k++) {
    if ($user[$k]{total_ISDN_time} > 0) {
      $ISDN_user_cnt++;
      $total_avg_ISDN_time += $user[$k]{total_ISDN_time};
    }
    if ($user[$k]{total_MODEM_time} > 0) {
      $MODEM_user_cnt++;
      $total_avg_MODEM_time += $user[$k]{total_MODEM_time};
    }
  }
  $BOTH_user_cnt = ($MODEM_user_cnt + $ISDN_user_cnt) - $usr_id;

  # get final average for ISDN
  if ($ISDN_user_cnt!=0) {
    $total_avg_ISDN_time = int ($total_avg_ISDN_time / ($ISDN_user_cnt*60));
  }
  if ($total_avg_ISDN_time >= 60 ) {
    $total_avg_ISDN_time = int ($total_avg_ISDN_time / 60);
    $total_avg_ISDN_time .= " hour" . ($total_avg_ISDN_time>1 ? "s" : "");
  } else { $total_avg_ISDN_time .= " minutes"; }

  # get final average for MODEM
  if ($MODEM_user_cnt!=0) {
    $total_avg_MODEM_time = int ($total_avg_MODEM_time / ($MODEM_user_cnt*60));
  }
  if ($total_avg_MODEM_time >= 60 ) {
    $total_avg_MODEM_time = int ($total_avg_MODEM_time / 60);
    $total_avg_MODEM_time .= " hour" . ($total_avg_MODEM_time>1 ? "s" : "");
  } else { $total_avg_MODEM_time .= " minutes"; }
  
  # convert octets and total time
  $tot_in  = convert_size( $tot_in_bytes );
  $tot_out = convert_size( $tot_out_bytes );
  $tot_oct = convert_size( $tot_in_bytes+$tot_out_bytes );
  $tot_ISDN_time = int ($tot_ISDN_time / 3600);
  $tot_MODEM_time = int ($tot_MODEM_time / 3600);
  $tot_time_together = $tot_ISDN_time + $tot_MODEM_time;
  $tot_bundled = int ($tot_bundled / 3600);

  # init the drawing
  draw_init_hourly($NAS_IPs[$current_NAS],$current_NAS, "normal");
  local $gfx_horiz;
  local $gfx_vert_new;
  local $gfx_vert_last1 = 500;
  local $gfx_vert_last2 = 500;
  local $gfx_vert_last3 = 500;

  $overload_MODEMS = "";
  $overload_ISDN = "";
  $total_ISDN_ports = $TOTAL_PORTS - $TOTAL_ANALOG;
  datafile_addtag("$current_NAS-hour-graph");
  for (local $i=0; $i < 24; $i++) {
    local @arr = split /\d+ /, $hour_port[0][$i];
    local $num1 = $#arr + 1;
    if ($num1 eq $TOTAL_ANALOG) { $overload_MODEMS .= "$i,"; }
    @arr = split /\d+ /, $hour_port[1][$i];
    local $num2 = $#arr + 1;
    if ($num2 eq $total_ISDN_ports) { $overload_ISDN .= "$i,"; }
    local $num3 = $num1+$num2;

    $mergeGraph[$current_NAS][$i] = $num3;

    datafile_put("$num1,$num2");
    if ($i != 23) { datafile_put(":"); };
    # draw it
    if ($i < 10) { $gfx_horiz = 40+18*$i-18; }
    else { $gfx_horiz = 43+18*$i-18; }

    # analog
    $gfx_vert_new = 278-$num1*3;
    if ($gfx_vert_last1!=500) {
      $im->line($gfx_horiz, $gfx_vert_last1, 
		$gfx_horiz+18, $gfx_vert_new, $red);
    }
    $gfx_vert_last1 = $gfx_vert_new;

    # isdn
    $gfx_vert_new = 278-$num2*3;
    if ($gfx_vert_last2!=500) {
      $im->line($gfx_horiz, $gfx_vert_last2, 
		$gfx_horiz+18, $gfx_vert_new, $green);
    }
    $gfx_vert_last2 = $gfx_vert_new;

    # total
    $gfx_vert_new = 278-$num3*3;
    if ($gfx_vert_last3!=500) {
      $im->line($gfx_horiz, $gfx_vert_last3, 
		$gfx_horiz+18, $gfx_vert_new, $blue);
    }
    $gfx_vert_last3 = $gfx_vert_new;
  }
  draw_end_hourly();
  datafile_endtag();

  local @arr = split /,/, $overload_MODEMS;
  local $MODEM_busyT = 0;
  foreach $chk (@arr) {
    $MODEM_busyT += inspect_hour ($chk, "Async", 
				  $current_NAS, $TOTAL_ANALOG);
  }

  local @arr = split /,/, $overload_ISDN;
  local $ISDN_busyT = 0;
  foreach $chk (@arr) {
    $ISDN_busyT += inspect_hour ($chk, "ISDN-Sync", 
				 $current_NAS, $total_ISDN_ports);
  }

  datafile_addtag("$current_NAS-no-users");
  datafile_put("$usr_id");
  datafile_endtag();
  datafile_addtag("$current_NAS-no-bytes");
  datafile_put("${tot_oct}:${tot_in}:${tot_out}");
  datafile_endtag();
  datafile_addtag("$current_NAS-no-hours");
  datafile_put("$tot_time_together");
  datafile_endtag();
  datafile_addtag("$current_NAS-no-users-both");
  datafile_put("$BOTH_user_cnt");
  datafile_endtag();
  datafile_addtag("$current_NAS-isdn-users");
  datafile_put("$ISDN_user_cnt");
  datafile_endtag();
  datafile_addtag("$current_NAS-isdn-hours");
  datafile_put("$tot_ISDN_time:$tot_bundled");
  datafile_endtag();
  datafile_addtag("$current_NAS-isdn-avg");
  datafile_put("$total_avg_ISDN_time");
  datafile_endtag();
  datafile_addtag("$current_NAS-isdn-busy");
  datafile_put("$ISDN_busyT");
  datafile_endtag();

  # check for strange ISDN dialups
  @arr = split / /, $ISDN_suspects;
  local $isdn_idx = 0;
  for (local $i=0; $i < ($#arr+1); $i++) {
    local @tmp = split /-/, $arr[$i];
    local $counter = 0;
    local $firstone = -1;
    local $second_one = -1;
    while ($counter != $isdn_idx) {
      if ($tmptable[$counter]{id} == $tmp[0]) { $firstone = $counter; }
      if ($tmptable[$counter]{id} == $tmp[1]) { $second_one = $counter; }
      if (($firstone != -1) && ($second_one != -1)) { last; }
      $counter++;
    }
    if ($firstone == -1) {
      $tmptable[$isdn_idx]{id}  = $tmp[0];
      $tmptable[$isdn_idx]{val} = 1;
      $tmptable[$firstone]{others} = "$tmp[1] "; 
      $isdn_idx++;
    } else { 
      $tmptable[$firstone]{others} .= "$tmp[1] "; 
      $tmptable[$firstone]{val}++; 
    }
    if ($second_one == -1) {
      $tmptable[$isdn_idx]{id}  = $tmp[1];
      $tmptable[$isdn_idx]{val} = 1;
      $tmptable[$firstone]{others} = "$tmp[0] "; 
      $isdn_idx++;
    } else {
      $tmptable[$second_one]{others} .= "$tmp[0] "; 
      $tmptable[$second_one]{val}++; 
    }
  }

  datafile_addtag("$current_NAS-isdn-suspected");
  local $msg = "";
  local $check = 0;
  for (local $counter=0; $counter < $isdn_idx; $counter++) {
    if ($tmptable[$counter]{val} > 1) {
      @arr = split / /, $tmptable[$counter]{others};
      local $p1 = $dat[$arr[0]]{'phone'};
      local $p2 = $dat[$arr[1]]{'phone'};
      local $p3 = $dat[$tmptable[$counter]{id}]{'phone'};

      if ( (defined $p2) && (defined $p3) && (defined $p1) ) {
	local $equals = 0;
	if ($p1 == $p2) { $equals++; }
	if ($p1 == $p3) { $equals++; }
	if ($p2 == $p3) { $equals++; }
	if ($equals==0) {
	  local $links = $tmptable[$counter]{val} + 1;
	  datafile_put("($links links) $dat[$arr[0]]{'user'}, end $dat[$arr[0]]{'stop_time'}, $dat[$arr[1]]{'stop_time'}, $dat[$tmptable[$counter]{id}]{'stop_time'}%");
	  $msg = "$msg\n\t($links links) $dat[$arr[0]]{'user'}, end $dat[$arr[0]]{'stop_time'}, $dat[$arr[1]]{'stop_time'}, $dat[$tmptable[$counter]{id}]{'stop_time'}";
	  $check = 1;
	}
      }
    }
  }
  if ($check==0) { datafile_put("(none)"); } 
  else {
    $msg = "Suspected ISDN dialups are listed below:$msg";
    mailto( "Dialup Stat - Suspected ISDN usage", $msg);
    dd "Warning for suspected ISDN dialups sent off.";
  }
  datafile_endtag();

  datafile_addtag("$current_NAS-analog-users");
  datafile_put("$MODEM_user_cnt");
  datafile_endtag();
  datafile_addtag("$current_NAS-analog-hours");
  datafile_put("$tot_MODEM_time");
  datafile_endtag();
  datafile_addtag("$current_NAS-analog-dialups");
  datafile_put("$MODEM_dialups");
  datafile_endtag();
  datafile_addtag("$current_NAS-analog-avg");
  datafile_put("$total_avg_MODEM_time");
  datafile_endtag();
  datafile_addtag("$current_NAS-analog-busy");
  datafile_put("$MODEM_busyT");
  datafile_endtag();

  datafile_addtag("$current_NAS-analog-suspected");
  local $msg = "";
  @arr = split / /, $MODEM_suspects;
  local $check = 0;
  for (local $i=0; $i < ($#arr+1); $i++) {
    local @tmp = split /-/, $arr[$i];
    local $startA = $dat[$tmp[1]]{'start_time'};
    local $stopA  = $dat[$tmp[1]]{'stop_time'};
    local $startB = $dat[$tmp[0]]{'start_time'};
    local $stopB  = $dat[$tmp[0]]{'stop_time'};
    local $name   = $dat[$tmp[1]]{'user'};
    $check = 1;
    if ( (diff_time($startB, $stopA)>$SIMULTANEOUS_THRESHOLD) &&
	 (diff_time($startB, $startA)>60) &&
         (diff_time($stopB, $stopA)>60) ) {
      datafile_put("(serious) $name, $startA-$stopA, $startB-$stopB%");
      $msg = "$msg\n\t(serious) $name, $startA-$stopA, $startB-$stopB";
    } else {
      datafile_put("(ignore) $name, $startA-$stopA, $startB-$stopB%");
    }
  }
  if ($check==0) { datafile_put("(none)"); }
  if ($msg ne "") {
    $msg = "Same MODEM users connected simultaneously listed below:$msg";
    mailto( "Dialup Stat - Suspected MODEM usage", $msg);
    dd "Warning for suspected MODEMS dialups sent off.";
  }

  datafile_endtag();

  # init the drawing
  draw_init_modem_usage($NAS_IPs[$current_NAS],$current_NAS, "ord", "");
  local $graph_left;
  local $graph_top;

  datafile_addtag("$current_NAS-analog-modem-cnt");
  datafile_put("$TOTAL_ANALOG");
  datafile_endtag();

  datafile_addtag("$current_NAS-analog-graph");
  for (local $i=1; $i <= $TOTAL_ANALOG; $i++) {
    local $usage = int ($MODEM_usage[$i] / 864);

    datafile_put("$usage");
    if ($i != $TOTAL_ANALOG) { datafile_put(":"); }
    if ($i < 10) { $graph_left = 17+18*$i }
    else { $graph_left = 20+18*$i }
    $graph_top = int ( 279 - $usage*2.4);

    $im->filledRectangle($graph_left, $graph_top, 
			 $graph_left+10, 276, gdTiled); 
    $im->rectangle($graph_left, $graph_top-1,
		   $graph_left+11, 278, $black); 
  }
  datafile_endtag();
  draw_end_modem_usage();
}

#----------------------------------------------
# parse
#===============================================
sub parse {
  $dent = 0;
  $usr_id = 0;
  $number_NAS_IPs = 0;

  if (-d "$LOGFILE1") {
    $LOGFILE1 =~ /.*(.)$/;
    if ($1 =~ /\//) { $LOGFILE1 .= "detail"; }
    else { $LOGFILE1 .= "/detail"; }
  }

  local $tmpval = $LOGFILE1;
  $tmpval =~ /(.+)\.(\d+)|(.+)/;
  local $nameMain = $1;
  local $nameTail = $2;
  $logfile2 = "";
  $logfile3 = "";
  if ((defined $nameMain) && ($nameMain ne "")) {
    $nameTail++; 
    $logfile2 = $nameMain . "." . $nameTail;
    if ($nameTail ne 0) { 
      if ($nameTail eq 1) {
	$logfile3 = $nameMain;
      } else {
	$nameTail -= 2;
	$logfile3 = $nameMain . "." . $nameTail;
      }
    } else { 
      $logfile3 = $nameMain;
    }
  } else {
    $logfile2 = $LOGFILE1 . ".0";
  }

  dd "Loading logfile $LOGFILE1";
  load_file ($LOGFILE1, "init");
  if ($logfile2 ne "") {
    dd "Loading logfile $logfile2";
    load_file ($logfile2, "normal");
  }
  if ($logfile3 ne "") {
    dd "Loading logfile $logfile3";
    load_file ($logfile3, "extra");
  }
  dd "Alright, found $dent entries there.";
  dd "Date of log entries: $date.";

  open_data_file;

  # now, list different NAS-IP addresses
  dd "Found $number_NAS_IPs different NAS-IP, listed below:";
  datafile_addtag("nas-ip");
  for (local $i=0; $i < ($number_NAS_IPs); $i++) {
    datafile_put("$NAS_IPs[$i]:");
    dd "\tNAS-IP $i -> $NAS_IPs[$i]";
  }
  datafile_endtag();

  # then, the show 
  for (my $i=0; $i < ($number_NAS_IPs); $i++) {
    main_per_nas $i;
  }

  if ($number_NAS_IPs>1) {
    dd "Merging NAS's hourly usage together to one graph.";
    local $allNas = "";
    for ($j=0; $j < $number_NAS_IPs; $j++) {
      $allNas .= "$NAS_IPs[$j],";
      $gfx_vert_new[$j] = 0;
      $gfx_vert_last[$j] = 500;
    }    
    draw_init_hourly($allNas,"sum", "extended");
    local $gfx_horiz;

    $total_ISDN_ports = $TOTAL_PORTS - $TOTAL_ANALOG;
    for (local $i=0; $i < 24; $i++) {
      for ($j=0; $j < $number_NAS_IPs; $j++) {
	
	local $num3 = $mergeGraph[$j][$i];
	# draw it
	if ($i < 10) { $gfx_horiz = 40+18*$i-18; }
	else { $gfx_horiz = 43+18*$i-18; }
	
	# total
	$gfx_vert_new[$j] = 278-$num3*3;
	if ($gfx_vert_last[$j]!=500) {
	  $im->line($gfx_horiz, $gfx_vert_last[$j], 
		    $gfx_horiz+18, $gfx_vert_new[$j], 
		    $colorArray[$j % 8]);
	}
	$gfx_vert_last[$j] = $gfx_vert_new[$j];
      }
    }
    draw_end_hourly();
  }

}

#----------------------------------------------
# Draw graph HOURLY init
#===============================================
sub draw_init_hourly {
  local ($nas,$nno,$mode) = @_;
  $graph_current_name = $nno;
  open (INIT_HOURLY, $GRAPH_INITIAL_HOURLY) or err(11,"$GRAPH_INITIAL_HOURLY");
  $im = newFromGif GD::Image (INIT_HOURLY);
  close INIT_HOURLY;

  if ($mode =~ /extended/) {
    $colorArray[0] = $im->colorAllocate(000, 000, 000);
    $colorArray[1] = $im->colorAllocate(255, 000, 000);
    $colorArray[2] = $im->colorAllocate(000, 000, 255);
    $colorArray[3] = $im->colorAllocate(250, 050, 250);
    $colorArray[4] = $im->colorAllocate(140, 140, 140);
    $colorArray[5] = $im->colorAllocate(000, 255, 255);
    $colorArray[6] = $im->colorAllocate(255, 128, 000);
    $colorArray[7] = $im->colorAllocate(000, 255, 000);
  }

  ($white,$black,$red,$blue,$purple,$gray,$green) = 
    (
     $im->colorAllocate(255, 255, 255),
     $im->colorAllocate(0, 0, 0),
     $im->colorAllocate(255, 0, 0),
     $im->colorAllocate(0,0,255),
     $im->colorAllocate(250,50,250),
     $im->colorAllocate(140,140,140),
     $im->colorAllocate(0,180,0)
    );

  if ($nno =~ /sum/) {
    $im->transparent($green);
    $im->filledRectangle(480,80, 540,180, $im->colorAllocate(254,255,255));
  }

  $im->transparent($white);       # white color is transparent

  # print message to where statistics are taken from
  $im->string(gdTinyFont,192,20,$GRAPH_SIGNATURE,$black);
  $im->string(gdSmallFont,450,6,"$date",$purple);

  if ($mode =~ /extended/) {

    local @nasser = split /,/, $nas;
    local $nasse = shift @nasser;
    local $countie = 0;
    while ($nasse ne "") {
      $im->string(gdSmallFont,462,17+$countie*10,"$nasse",
		  $colorArray[$countie % 8]);
      $countie++;
      $nasse = shift @nasser || last;
    }

  } else { $im->string(gdSmallFont,462,17,"$nas",$gray); }

  # draw horizontal numbers and lines
  for (local $i=0; $i < 24; $i++) {
    $im->string(gdSmallFont,38+18*$i,282,"$i",$blue);
    if ($i < 10) { $im->line(40+18*$i,276,40+18*$i,281,$black); }
    else { $im->line(43+18*$i,276,43+18*$i,281,$black); }
  }

  # draw vertical numbers and lines
  local $pos = 271-15;
  local $cnt = 5;
  for (local $i=0; $i < 16; $i++) {
    if ($cnt > $TOTAL_PORTS) { $colNumber = $gray; }
    else { $colNumber = $blue; }
    if ($cnt < 10) { $im->string(gdSmallFont,19,$pos,"$cnt",$colNumber); }
    else { $im->string(gdSmallFont,13,$pos,"$cnt",$colNumber); }
    $im->line(28,$pos+7,32,$pos+7,$black);
    $pos -= 15;
    $cnt += 5;
  }
  
  # draw max ports line
  $pos = 278-$TOTAL_PORTS*3;
  $im->dashedLine(38,$pos,475,$pos,$gray);

  if ($TOTAL_PORTS > 80) {
    dd "Warning! Max ports is over 80 ports. Adjust code.";
  }
}  

#----------------------------------------------
# Draw graph end
#===============================================
sub draw_end_hourly {

  ${rdate} =~ /(\d+)-(\d+)-(\d+)/;
  local $tmpyr = $1;  
  if (!(-e "${GFXDIR}$tmpyr")) {
    dd "Creating gfx dir $tmpyr.";
    mkdir "${GFXDIR}$tmpyr", -1;
  }

open (OUTPUT, ">${GFXDIR}$tmpyr/${rdate}_$graph_current_name-hourly_usage.gif") or
      err(12, "${GFXDIR}$tmpyr/${rdate}_$graph_current_name-hourly_usage.gif");
  binmode STDOUT;
  print OUTPUT $im->gif;
  close OUTPUT;

  dd "Created 'Hourly usage report' GIF"; 
}

#----------------------------------------------
# Draw graph ANALOG MODEM USAGE init
#===============================================
sub draw_init_modem_usage {
  local ($nas,$nno,$mode,$modval) = @_;
  $graph_current_name = $nno;

  if ($mode =~ /week/) {
    open (INIT_ANALOG, $GRAPH_INITIAL_WEEKLY_ANALOG) or
      err(11, "$GRAPH_INITIAL_WEEKLY_ANALOG"); }
  elsif ($mode =~ /month/) {
    open (INIT_ANALOG, $GRAPH_INITIAL_MONTHLY_ANALOG) or
      err(11, "$GRAPH_INITIAL_MONTHLY_ANALOG"); }
  else { open (INIT_ANALOG, $GRAPH_INITIAL_ANALOG) or
      err(11, "$GRAPH_INITIAL_ANALOG"); }
  $im = newFromGif GD::Image (INIT_ANALOG);
  close INIT_ANALOG;

  ($white,$black,$red,$blue,$purple,$gray,$green) = 
    (
     $im->colorAllocate(255, 255, 255),
     $im->colorAllocate(0, 0, 0),
     $im->colorAllocate(255, 0, 0),
     $im->colorAllocate(0,0,255),
     $im->colorAllocate(250,50,250),
     $im->colorAllocate(140,140,140),
     $im->colorAllocate(0,180,0)
    );
  $im->transparent($white);       # white color is transparent

  # print message to where statistics are taken from
  if ($mode =~ /week/) { 
    $im->string(gdTinyFont,94,20,$GRAPH_SIGNATURE,$black);
    $im->string(gdSmallFont,390,6,"$modval",$purple); 
  } elsif ($mode =~ /month/) {
    $im->string(gdTinyFont,94,20,$GRAPH_SIGNATURE,$black);
    $im->string(gdSmallFont,492,6,"$modval",$purple); 
  } else { 
    $im->string(gdSmallFont,450,6,"$date",$purple); 
    $im->string(gdTinyFont,192,20,$GRAPH_SIGNATURE,$black);
  }
  $im->string(gdSmallFont,462,17,"$nas",$gray);

  # draw horizontal numbers and lines
  if ($TOTAL_ANALOG > 24) {
    dd "Warning! Total analog ports > 24. Not supported yet.";
  }

  for (local $i=1; $i < ($TOTAL_ANALOG+1); $i++) {
    $im->string(gdSmallFont,20+18*$i,282,"$i",$blue);
    if ($i < 10) { $im->line(22+18*$i,276,22+18*$i,281,$black); }
    else { $im->line(25+18*$i,276,25+18*$i,281,$black); }
  }

  # draw vertical numbers and lines
  local $pos = 271-12;
  local $cnt = 5;
  for (local $i=0; $i < 20; $i++) {
    if (($cnt > 5) && (($cnt%2)!=1)) {
      if ($cnt > 95) { $im->string(gdSmallFont,8,$pos,"$cnt",$blue); }
      else { $im->string(gdSmallFont,13,$pos,"$cnt",$blue); }
    }
    $im->line(28,$pos+7,32,$pos+7,$black);
    $pos -= 12;
    $cnt += 5;
  }

  open(TILEGIF, $GRAPH_INITIAL_TILE) or err(11, "$GRAPH_INITIAL_TILE");
  $tile = newFromGif GD::Image(TILEGIF);
  close TILEGIF;
  $im->setTile($tile);
}  

#----------------------------------------------
# Draw graph ANALOG MODEM USAGE end
#===============================================
sub draw_end_modem_usage {

  ${rdate} =~ /(\d+)-(\d+)-(\d+)/;
  local $tmpyr = $1;  
  if (!(-e "${GFXDIR}$tmpyr")) {
    dd "Creating gfx dir $tmpyr.";
    mkdir "${GFXDIR}$tmpyr", -1;
  }

    open (OUTPUT, ">${GFXDIR}$tmpyr/${rdate}_$graph_current_name-analog_modem.gif") or
      err(12, "${GFXDIR}$tmpyr/${rdate}_$graph_current_name-analog_modem.gif");
  binmode STDOUT;
  print OUTPUT $im->gif;
  close OUTPUT;

  dd "Created 'Analog modem usage report' GIF"; 
}

#-----------------------------------------------
# Create the html
#===============================================
sub write_html {
    open DATAFILE, "${DATADIR}$rdate.data" or
      err(11, "${DATADIR}$rdate.data");

  $line = <DATAFILE>;
  local $current_nas = 0;
  local $curNAS_count = 0;

  ${rdate} =~ /(\d+)-(\d+)-(\d+)/;
  local $tmpyr = $1;  
  if (!(-e "${GFXDIR}$tmpyr")) {
    dd "Creating gfx dir $tmpyr.";
    mkdir "${GFXDIR}$tmpyr", -1;
  }

  $highestISDNport = 0;
  $highestMODEMport = 0;

  while ($line ne "") {
    $line =~ /<(.*)>((.+)|(\n.+))<\/>\n/;
    local $cmd = "$1 ";
    local $str = $2;

    if ($cmd =~ /date\ /) { 
      htmlfile_put("<h3>Date of log entry: $str</h3>"); 
      if ($number_NAS_IPs>1) {
	  htmlfile_put("<img src=\"${WEBROOT}${GFXDIRR}$tmpyr/${rdate}_sum-hourly_usage.gif\"><br>Graph showing total hour usage. Different color per NAS.");
	htmlfile_put("<h2>More detailed info per NAS follows below.</h2>"); 
      }
    } elsif ($cmd =~ /nas-ip\ /) {
      @nas_name = split /:/, $str;
    } elsif ($cmd =~ /\d+-no-users\ /) {
      htmlfile_put("<li>The service was used by $str different users.<br>");
    } elsif ($cmd =~ /(\d+)-hour-graph\ /) {

      local @dumbo = split /:/, $str;
      for (local $dumba=0; $dumba < 24; $dumba++) {
	  local @ears = split /,/, $dumbo[$dumba];
	  local $curnum1 = scalar $ears[0];
	  local $curnum2 = scalar $ears[1];
	  if ($curnum2 > ${highestISDNport}) {
	      $highestISDNport = $curnum2;
	  }
	  if ($curnum1 > ${highestMODEMport}) {
	      $highestMODEMport = $curnum1;
	  }
      }
      $current_nas = $1;
      $graph_current_name = $curNAS_count++;
      htmlfile_put("<hr>\n<h2>NAS-IP $nas_name[$current_nas]</h2>");
      htmlfile_put("<h3><b>General info</b></h3>");
      htmlfile_put("<img src=\"${WEBROOT}${GFXDIRR}$tmpyr/${rdate}_$graph_current_name-hourly_usage.gif\"><ul>");

    } elsif ($cmd =~ /\d+-no-bytes\ /) {
      local @tmp = split /:/, $str;
      htmlfile_put("<li>$tmp[0] passed through ($tmp[1] out, $tmp[2] in).<br>");
    } elsif ($cmd =~ /\d+-no-hours\ /) {
      htmlfile_put("<li>All together, the service was used for $str hours.<br>");
    } elsif ($cmd =~ /\d+-no-users-both\ /) {
      htmlfile_put("<li>Number of users using both ISDN and MODEM: $str.<br>");
    } elsif ($cmd =~ /\d+-isdn-users\ /) {
      htmlfile_put("</ul>\n<h3>ISDN</h3><ul>");
      htmlfile_put("<li>$str different users used the ISDN service.<br>");
    } elsif ($cmd =~ /\d+-isdn-hours\ /) {
      local @tmp = split /:/, $str;
      htmlfile_put("<li>The ISDN service was in use for $tmp[0] hours, including
around $tmp[1] hours of 128 bit ISDN connections.<br>");
    } elsif ($cmd =~ /\d+-isdn-avg\ /) {
      htmlfile_put("<li>Maximum ISDN ports in use during period was ${highestISDNport}.<br>");
      htmlfile_put("<li>Average connection time per ISDN user was $str.<br>");
    } elsif ($cmd =~ /\d+-isdn-busy\ /) {
      if ($str ne 0) {
	htmlfile_put("<li>All B-channels were busy for $str seconds.<br>");
      }
    } elsif ($cmd =~ /\d+-analog-users\ /) {
      htmlfile_put("</ul>\n<h3>MODEM</h3>");
      htmlfile_put("<img src=\"${WEBROOT}${GFXDIRR}$tmpyr/${rdate}_$graph_current_name-analog_modem.gif\"><br><ul>");
      htmlfile_put("<li>$str different users used the MODEM service.<br>");
    } elsif ($cmd =~ /\d+-analog-hours\ /) {
      htmlfile_put("<li>The MODEM service was in use for $str hours.<br>");
    } elsif ($cmd =~ /\d+-analog-dialups\ /) {
      htmlfile_put("<li>There were $str MODEM dialups in this period.<br>");
    } elsif ($cmd =~ /\d+-analog-avg\ /) {
      htmlfile_put("<li>Maximum MODEMs in use during period was ${highestMODEMport}.<br>");
      htmlfile_put("<li>Average connection time per MODEM user was $str.<br>");
    } elsif ($cmd =~ /\d+-analog-busy\ /) {
      if ($str ne 0) {
	htmlfile_put("<li>All MODEMs were busy for $str seconds.<br>");
      }
      htmlfile_put("</ul>");
    }

    $line = <DATAFILE> || last;
  }
  close DATAFILE;
  
  dd "Written html file.";  
}

#-----------------------------------------------
# open_data_file
#===============================================
sub mailto {
  local ($subj, $msg) = @_;

  if ($WARNINGSENABLED eq "1") {
    open MAILFILE, ">${TMPDIR}mail.dat" or 
      err(12, "${TMPDIR}mail.dat");

    print MAILFILE "\n$date\n\n$msg\n\n--DialupStat $VERSION\n--University of Tromsoe, Norway\n";
    close MAILFILE;
    if ($USE_MAILX==1) {
      system( "$MAILXPATH $MAILSUB_OPTION \"$subj\" $WARNINGSEMAIL < 
${TMPDIR}mail.dat" );
    } else {
      system( "cat ${TMPDIR}mail.dat | $MAILPATH $MAILSUB_OPTION \"$subj\" $WARNINGSEMAIL" );
    }

    local @dellist = split /:/, "${TMPDIR}mail.dat:";
    unlink @dellist;
  }
}

#-----------------------------------------------
# Create Weekly Report
#===============================================
sub createWeeklyReport {
  local ($startWeek, $mode, $mYear, $mMon) = @_;

  local ($year, $mon, $day) = split /-/, "$startWeek-";
  local $found = 0;
  local $cnt   = 0;
  local $rday  = $day;
  local $rmon  = $mon;
  if ($mode =~ /norm/) {
    dd "Inspecting week of $year-$mon-$day...";

    while ($found == 0) {
      if (-r "${DATADIR}$year-$rmon-$rday.data") {
	open TESTFILE, "${DATADIR}$year-$rmon-$rday.data";
	$line = <TESTFILE>;
	close TESTFILE;
	
	if ($line =~ /Mon /) { $found = 1; last; }
      }
      
      if ($day==1) {
	if ($mon==1) {
	  $year--;
	  $mon = 12;
	  $rmon = $mon;
	} else { 
	  $mon--; 
	  if ($mon < 10) { $rmon = "0$mon"; }
	  else { $rmon = $mon; }
	}
	$day = $ndays{$rmon};
	$rday = $day;
	if ($mon==2) {
	  local $divtemp = $year % 4;
	  if ($divtemp==0) { $day++; $rday = $day; } # skudd-aar
	}
      } else {
	$day--; 
	if ($day < 10) { $rday = "0$day"; }
	else { $rday = $day; }
      }
      
      if ($cnt==6) { err(7, 0); }
      $cnt++;
    }
    
    dd "Week begins on $year-$rmon-$rday.";
  }
  $cnt = 0;

  if (!($mode =~ /month/)) {
    $maxDay = $ndays{$rmon};
  } else {
    $maxDay = $ndays{$mMon};
    $mon = $mMon;
    $year = $mYear;
  }
  if ($mon==2) {
    local $divtemp = $year % 4;
    if ($divtemp==0) { $maxDay++; } # skudd-aar
  }

  if (!($mode =~ /month/)) {
    while ($cnt!=7) {
      $week[$cnt] = "$year-$rmon-$rday";
      if ($day==$maxDay) {
	if ($mon==12) {
	  $year++;
	  $mon = 1;
	  $rmon = "01";
	} else { 
	  $mon++; 
	  if ($mon < 10) { $rmon = "0$mon"; }
	  else { $rmon = $mon; }
	}
	$day = 1;
	$rday = "01";
      } else {
	$day++; 
	if ($day < 10) { $rday = "0$day"; }
	else { $rday = $day; }
      }
      $cnt++;
    }
    # open data file
    open DATAFILE, ">${DATADIR}week_$week[0]_$week[6].data" or 
      err(11, "${DATADIR}week_$week[0]_$week[6].data");
  } else {
    open DATAFILE, ">${DATADIR}month_$mYear-$mMon.data" or
      err(12, "${DATADIR}month_$mYear-$mMon.data");
  } 

  datafile_addtag("date");
  if (!($mode =~ /month/)) {
    local @arr1 = split /-/, "$week[0]-";
    local @arr2 = split /-/, "$week[6]-";
    datafile_put("Mon $arr1[2] $RmonList{$arr1[1]} $arr1[0]:Sun $arr2[2] $RmonList{$arr2[1]} $arr2[0]");
  } else {
    datafile_put("$RmonList{$mMon} $mYear");
  }
  datafile_endtag();

  $cnt = 0;
  local $nas_ip_tag = "";
  local $ceiling_graph = 0;
  local $mDay = 1;
  local $terminal;
  if ($mode =~ /month/) { $terminal = $maxDay; $globMaxDay = $maxDay; }
  else { $terminal = 7; }

  while ($cnt!=$terminal) {
    local $realDay;
    if ($mDay < 10) { $realDay = "0$mDay"; }
    else { $realDay = "$mDay"; }
    $mDay++;	   

    if ((($mode =~ /month/) && 
	 (-r "${DATADIR}$mYear-$mMon-$realDay.data")) ||
	 ((!($mode =~ /month/)) && 
	 (-r "${DATADIR}$week[$cnt].data"))) {
      if ($mode =~ /month/) {
	open TESTFILE, "${DATADIR}$mYear-$mMon-$realDay.data";
      } else {
	open TESTFILE, "${DATADIR}$week[$cnt].data";
      }
      $line = <TESTFILE>;
      local $current_nas = 0;
      while ($line ne "") {
	$line =~ /<(.*)>((.+)|(\n.+))<\/>\n/;
	local $cmd = "$1 ";
	local $str = $2;

	if ($cmd =~ /nas-ip\ /) {
	  if ($nas_ip_tag eq "") { 
	    $nas_ip_tag = $str;
	    local @arr = split /:/, $str;
	    $number_nas = $#arr+1;
	    local $initer = 0;
	    dd "Initializing $number_nas NAS per day.";
	    while ($initer < $number_nas) {
	      for ($i=0; $i < $terminal; $i++) {
		$weekday[$initer][$i]{'hour-graph'} = "0,0";
	      }
	      $week[$initer]{'no-users'} = 0;
	      $week[$initer]{'no-bytes'} = "0.0 MB:0.0 MB:0.0 MB";
	      $week[$initer]{'no-hours'} = 0;
	      $week[$initer]{'no-users-both'} = 0;
	      $week[$initer]{'isdn-users'} = 0;
	      $week[$initer]{'isdn-hours'} = "0:0";
	      $week[$initer]{'isdn-avg'} = "0 hour";
	      $week[$initer]{'isdn-busy'} = 0;
	      $week[$initer]{'analog-users'} = 0;
	      $week[$initer]{'analog-hours'} = 0;
	      $week[$initer]{'analog-dialups'} = 0;
	      $week[$initer]{'analog-avg'} = 0;
	      $week[$initer]{'analog-busy'} = 0;
	      $modem_count[$initer] = 0;
	      
	      $initer++;
	    }
	  } 
	  else {

	  }
	} elsif ($cmd =~ /(\d+)-no-users\ /) {
	  $week[$1]{'no-users'} += $str;
	} elsif ($cmd =~ /(\d+)-hour-graph\ /) {
	  local $nas = $1;
	  $weekday[$nas][$cnt]{'hour-graph'} =~ /(\d+),(\d+)/;
	  local $modems = $1;
	  local $isdns  = $2;
	  local @arr = split /:/, "$str:";
	  for (local $j=0; $j < 24; $j++) {
	    local @small = split /,/, $arr[$j];
	    $modems += $small[0];
	    $isdns  += $small[1];
	  }

	  if ($modems > $ceiling_graph) { $ceiling_graph = $modems; }
	  if ($isdns  > $ceiling_graph) { $ceiling_graph = $isdns; }

	  $weekday[$nas][$cnt]{'hour-graph'} = "$modems,$isdns";
	} elsif ($cmd =~ /(\d+)-no-bytes\ /) {
	  local $nas = $1;
	  local @tmp2 = split /:/, $week[$nas]{'no-bytes'};
	  local @tmp1 = split /:/, $str;

	  $tmp2[0] =~ /(\d+.\d+)\ MB/;
	  $tmp2[0] = $1;
	  $tmp1[0] =~ /(\d+.\d+)\ MB/;
	  $tmp1[0] = $1;
	  local $val1 = $tmp2[0] + $tmp1[0];

	  $tmp2[1] =~ /(\d+.\d+)\ MB/;
	  $tmp2[1] = $1;
	  $tmp1[1] =~ /(\d+.\d+)\ MB/;
	  $tmp1[1] = $1;
	  local $val2 = $tmp2[1] + $tmp1[1];

	  $tmp2[2] =~ /(\d+.\d+)\ MB/;
	  $tmp2[2] = $1;
	  $tmp1[2] =~ /(\d+.\d+)\ MB/;
	  $tmp1[2] = $1;
	  local $val3 = $tmp2[2] + $tmp1[2];

	  $week[$nas]{'no-bytes'} = "$val1 MB:$val2 MB:$val3 MB";
	} elsif ($cmd =~ /(\d+)-no-hours\ /) {
	  $week[$1]{'no-hours'} += $str;
	} elsif ($cmd =~ /(\d+)-no-users-both\ /) {
	  $week[$1]{'no-users-both'} += $str;
	} elsif ($cmd =~ /(\d+)-isdn-users\ /) {
	  $week[$1]{'isdn-users'} += $str;
	} elsif ($cmd =~ /(\d+)-isdn-hours\ /) {
	  local $nas = $1;
	  $str =~ /(\d+):(\d+)/;
	  local $val1 = $1;
	  local $val2 = $2;
	  $week[$nas]{'isdn-hours'} =~ /(\d+):(\d+)/;
	  $val1 += $1;
	  $val2 += $2;
	  $week[$nas]{'isdn-hours'} = "$val1:$val2";	  
	} elsif ($cmd =~ /(\d+)-isdn-avg\ /) {
	  local $nas = $1;
	  $str =~ /(\d+)\ (.).*/;
	  if ($2 eq "h") {
	    $time_isdn[$nas] += $1*60;
	  } else { $time_isdn[$nas] += $1; }
	} elsif ($cmd =~ /(\d+)-isdn-busy\ /) {
	  $week[$1]{'isdn-busy'} += $str;
	} elsif ($cmd =~ /(\d+)-analog-users\ /) {
	  $week[$1]{'analog-users'} += $str;
	} elsif ($cmd =~ /(\d+)-analog-hours\ /) {
	  $week[$1]{'analog-hours'} += $str;
	} elsif ($cmd =~ /(\d+)-analog-dialups\ /) {
	  $week[$1]{'analog-dialups'} += $str;
	} elsif ($cmd =~ /(\d+)-analog-avg\ /) {
	  local $nas = $1;
	  $str =~ /(\d+)\ (.).*/;
	  if ($2 eq "h") {
	    $time_modem[$nas] += $1*60;
	  } else { $time_modem[$nas] += $1; }
	} elsif ($cmd =~ /(\d+)-analog-busy\ /) {
	  $week[$1]{'analog-busy'} += $str;
	} elsif ($cmd =~ /(\d+)-analog-modem-cnt\ /) {
	  local $nas = $1;
	  if ($modem_count[$nas]==0) {
	    $modem_count[$nas] = $str;
	    for (local $l=0; $l < $modem_count[$nas]; $l++) {
	      $weekday[$nas][$l]{'analog-graph'} = 0;
	    }
	  } else {
	    if ($modem_count[$1] ne $str) { err(9, "$week[$cnt].data"); }
	  }
	} elsif ($cmd =~ /(\d+)-analog-graph\ /) {
	  local $nas = $1;
	  local @arr = split /:/, "$str:";
	  for (local $l=0; $l < $modem_count[$nas]; $l++) {
	    $weekday[$nas][$l]{'analog-graph'} += $arr[$l];
	  }
	}

	$line = <TESTFILE> || last;
      }
      close TESTFILE;
    }
    
    $cnt++;
  }

  for ($i=0; $i < $number_nas; $i++) {
    datafile_addtag("nas-ip");
    datafile_put("$nas_ip_tag");
    datafile_endtag();

    datafile_addtag("$i-dgraph-y");
    datafile_put("$ceiling_graph");
    datafile_endtag();

    datafile_addtag("$i-day-graph");
    local $nonTerminal = $terminal - 1;
    local $majorString = "";
    for (local $k=0; $k < $terminal; $k++) {
      $majorString .= $weekday[$i][$k]{'hour-graph'};
      if ($k!=$nonTerminal) { $majorString .= ":"; }
    }
    datafile_put("$majorString");
    datafile_endtag();

    local $tmp = int ($week[$i]{'no-users'} / $terminal);
    datafile_addtag("$i-no-users");
    datafile_put("$tmp");
    datafile_endtag();

    $week[$i]{'no-bytes'} =~ /(\d+.\d+)\ MB:(\d+.\d+)\ MB:(\d+.\d+)\ MB/;
    local $val1 = convert_size_GB($1);
    local $val2 = convert_size_GB($2);
    local $val3 = convert_size_GB($3);
    datafile_addtag("$i-no-bytes");
    datafile_put("$val1:$val2:$val3");
    datafile_endtag();

    datafile_addtag("$i-no-hours");
    datafile_put("$week[$i]{'no-hours'}");
    datafile_endtag();

    local $tmp = int ($week[$i]{'no-users-both'} / $terminal);
    datafile_addtag("$i-no-users-both");
    datafile_put("$tmp");
    datafile_endtag();

    local $tmp = int ($week[$i]{'isdn-users'} / $terminal);
    datafile_addtag("$i-isdn-users");
    datafile_put("$tmp");
    datafile_endtag();

    datafile_addtag("$i-isdn-hours");
    datafile_put("$week[$i]{'isdn-hours'}");
    datafile_endtag();

    local $tmp = int ($time_isdn[$i] / $terminal);
    if ($tmp > 59) {
      $tmp = int ($tmp/60);
      $tmp .= $tmp>1 ? "hours" : " hour";
    } else { $tmp .= " minutes"; }
    datafile_addtag("$i-isdn-avg");
    datafile_put("$tmp");
    datafile_endtag();

    datafile_addtag("$i-isdn-busy");
    datafile_put("$week[$i]{'isdn-busy'}");
    datafile_endtag();

    local $tmp = int ($week[$i]{'analog-users'} / $terminal);
    datafile_addtag("$i-analog-users");
    datafile_put("$tmp");
    datafile_endtag();

    datafile_addtag("$i-analog-hours");
    datafile_put("$week[$i]{'analog-hours'}");
    datafile_endtag();

    datafile_addtag("$i-analog-dialups");
    datafile_put("$week[$i]{'analog-dialups'}");
    datafile_endtag();

    local $tmp = int ($time_modem[$i] / $terminal);
    if ($tmp > 59) {
      $tmp = int ($tmp/60);
      $tmp .= $tmp>1 ? "hours" : " hour";
    } else { $tmp .= " minutes"; }
    datafile_addtag("$i-analog-avg");
    datafile_put("$tmp");
    datafile_endtag();
    
    datafile_addtag("$i-analog-busy");
    datafile_put("$week[$i]{'analog-busy'}");
    datafile_endtag();
    
    datafile_addtag("$i-analog-modem-cnt");
    datafile_put("$modem_count[$i]");
    datafile_endtag();

    datafile_addtag("$i-analog-graph");    
    local $majorString = "";
    local $l = 0;
    while ($l < $modem_count[$i]) {
      local $tmp = int ($weekday[$i][$l]{'analog-graph'} / $terminal);
      $majorString .= "$tmp";
      $l++;
      if ($l!=$modem_count[$i]) { $majorString .= ":"; }
    }
    datafile_put($majorString);
    datafile_endtag();
  }     
  close DATAFILE;

  if (!($mode =~ /month/)) {
    createWeekHTML("week_$week[0]_$week[6]", "norm");
  } else { createWeekHTML("month_$mYear-$mMon", "monthly"); }
}

#-----------------------------------------------
# Create Weekly Report Html
#===============================================
sub createWeekHTML {
  local ($datafile, $mode) = @_;

  local $tmpyr;
  if ($mode =~ /monthly/) {
      ${datafile} =~ /.*_(\d+)-\d+.*/;
      $tmpyr = $1;
  } else {
      ${datafile} =~ /(\d+)-(\d+)-(\d+)/;
      $tmpyr = $1;
  }

  if (!(-e "${GFXDIR}$tmpyr")) {
    dd "Creating gfx dir $tmpyr.";
    mkdir "${GFXDIR}$tmpyr", -1;
  }
  if (!(-e "${HTMLDIR}$tmpyr")) {
    dd "Creating html dir $tmpyr.";
    mkdir "${HTMLDIR}$tmpyr", -1;
  }

  local $labbie;
  if ($mode =~ /monthly/) { 
    $labbie = "month"; 
    open_html_file("${HTMLDIR}$tmpyr/${datafile}.html", 
		   "${HTML_INITIAL_HEADING_MONTH}");
  }
  else { 
    $labbie = "week"; 
    open_html_file("${HTMLDIR}$tmpyr/${datafile}.html", 
		   "${HTML_INITIAL_HEADING_WEEK}");
  }

  open DATAFILE, "${DATADIR}$datafile.data" or
    err(11, "${DATADIR}$datafile.data");
  $line = <DATAFILE>;
  local $current_nas = 0;
  while ($line ne "") {
    $line =~ /<(.*)>((.+)|(\n.+))<\/>\n/;
    local $cmd = "$1 ";
    local $str = $2;

    if ($cmd =~ /date\ /) { 
      if (!($mode =~ /monthly/)) {
	local @arr = split /:/,"$str:";
	$beginDate = $arr[0];
	$endDate = $arr[1];
	htmlfile_put("<h3>Week of log entry: $arr[0] - $arr[1]</h3>"); 
      } else {
	$beginDate = $str;
	htmlfile_put("<h3>Month of log entry: $str</h3>"); 
      }
    } elsif ($cmd =~ /nas-ip\ /) {
      @nas_name = split /:/, $str;
    } elsif ($cmd =~ /\d+-no-users\ /) {
      htmlfile_put("<li>The service was used by $str different users on average per day.<br>");
    } elsif ($cmd =~ /(\d+)-day-graph\ /) {
      $current_nas = $1;
      htmlfile_put("<hr>\n<h2>NAS-IP $nas_name[$current_nas]</h2>");
      htmlfile_put("<h3><b>General info</b></h3>");
      htmlfile_put("<img src=\"${WEBROOT}${GFXDIRR}$tmpyr/${datafile}_${current_nas}-A.gif\"><ul>");
      $day_graph = $str;
    } elsif ($cmd =~ /\d+-no-bytes\ /) {
      local @tmp = split /:/, $str;
      htmlfile_put("<li>$tmp[0] passed through ($tmp[1] out, $tmp[2] in) during the $labbie.<br>");
    } elsif ($cmd =~ /\d+-no-hours\ /) {
      htmlfile_put("<li>All together for the $labbie, the service was used for $str hours.<br>");
    } elsif ($cmd =~ /\d+-no-users-both\ /) {
      htmlfile_put("<li>Number of users on the average using both ISDN and MODEM per day: $str.<br>");
    } elsif ($cmd =~ /\d+-isdn-users\ /) {
      htmlfile_put("</ul>\n<h3>ISDN</h3><ul>");
      htmlfile_put("<li>$str different users on the average used the ISDN service per day.<br>");
    } elsif ($cmd =~ /\d+-isdn-hours\ /) {
      local @tmp = split /:/, $str;
      htmlfile_put("<li>The ISDN service was in use for $tmp[0] hours during the $labbie, including around $tmp[1] hours of 128 bit ISDN connections.<br>");
    } elsif ($cmd =~ /\d+-isdn-avg\ /) {
      htmlfile_put("<li>Average connection time per ISDN user was $str.<br>");
    } elsif ($cmd =~ /\d+-isdn-busy\ /) {
      if ($str ne 0) {
	htmlfile_put("<li>All B-channels were busy for $str seconds throughout the $labbie.<br>");
      }
    } elsif ($cmd =~ /\d+-analog-users\ /) {
      htmlfile_put("</ul>\n<h3>MODEM</h3>");
      htmlfile_put("<img src=\"${WEBROOT}${GFXDIRR}$tmpyr/${datafile}_${current_nas}-B.gif\"><br><ul>");
      htmlfile_put("<li>$str different users on the average used the MODEM service per day.<br>");
    } elsif ($cmd =~ /\d+-analog-hours\ /) {
      htmlfile_put("<li>The MODEM service was in use for $str hours during the $labbie.<br>");
    } elsif ($cmd =~ /\d+-analog-dialups\ /) {
      htmlfile_put("<li>There were $str MODEM dialups in this period.<br>");
    } elsif ($cmd =~ /\d+-analog-avg\ /) {
      htmlfile_put("<li>Average connection time per MODEM user was $str.<br>");
    } elsif ($cmd =~ /\d+-dgraph-y\ /) {
      $dgraph_y = $str;
    } elsif ($cmd =~ /\d+-analog-modem-cnt\ /) {
      $analog_cnt = $str;
    } elsif ($cmd =~ /(\d+)-analog-graph\ /) {
      $analog_graph = $str;
      local $nas = $nas_name[$1];

      if ($mode =~ /monthly/) {
	local $date = $beginDate;
	createWeek_DayGfx(   "${GFXDIR}$tmpyr/${datafile}_${current_nas}-A.gif",
			     "${dgraph_y}", "${day_graph}", $nas, $date,
			     "monthly");
	createWeek_ModemGfx( "${GFXDIR}$tmpyr/${datafile}_${current_nas}-B.gif",
			     "${analog_cnt}", "${analog_graph}", $nas, 
			     $date, "monthly" );
	
      } else {
	$beginDate =~ /\w+\ (.+)/;
	$beginDate = $1;
	$endDate =~ /\w+\ (.+)/;
	$endDate = $1;
	local $date = "$beginDate - $endDate";      
	# now create graphs
	createWeek_DayGfx(   "${GFXDIR}$tmpyr/${datafile}_${current_nas}-A.gif",
			     "${dgraph_y}", "${day_graph}", $nas, $date,
			     "ord");
	createWeek_ModemGfx( "${GFXDIR}$tmpyr/${datafile}_${current_nas}-B.gif",
			     "${analog_cnt}", "${analog_graph}", $nas, 
			     $date, "ord" );
      }
    } elsif ($cmd =~ /\d+-analog-busy\ /) {
      if ($str ne 0) {
	htmlfile_put("<li>All MODEMs were busy for $str seconds throughout the $labbie.<br>");
      }
      htmlfile_put("</ul>");
    }

    $line = <DATAFILE> || last;
  }
  close DATAFILE;
  
  dd "Written html file.";  
  
  if ($mode =~ /monthly/) {
    close_html_file("$HTML_INITIAL_TAIL_WEEK");
  } else {
    close_html_file("$HTML_INITIAL_TAIL_MONTH");
  }
}

#-----------------------------------------------
# Create Weekly Report Graphics *weekday*
#===============================================
sub createWeek_DayGfx {
  local ($gfxDst, $yMax, $data, $nas, $date, $mode) = @_;

  if ($mode =~ /monthly/) {
    open (INIT_WEEKLY, $GRAPH_INITIAL_MONTHLY) or
      err(11, "$GRAPH_INITIAL_MONTHLY");
  } else {
    open (INIT_WEEKLY, $GRAPH_INITIAL_WEEKLY) or
      err(11, "$GRAPH_INITIAL_WEEKLY");
  }
  $im = newFromGif GD::Image (INIT_WEEKLY);
  close INIT_WEEKLY;

  ($white,$black,$red,$blue,$purple,$gray,$green) = 
    (
     $im->colorAllocate(255, 255, 255),
     $im->colorAllocate(0, 0, 0),
     $im->colorAllocate(255, 0, 0),
     $im->colorAllocate(0,0,255),
     $im->colorAllocate(250,50,250),
     $im->colorAllocate(140,140,140),
     $im->colorAllocate(0,180,0)
    );
  $im->transparent($white);       # white color is transparent

  # print message to where statistics are taken from
  $im->string(gdTinyFont,165,20,$GRAPH_SIGNATURE,$black);

  local $offset = 55;
  local $terminal;
  if ($mode =~ /monthly/) {
    $im->string(gdSmallFont,491,6,"$date",$purple);
    $terminal = $globMaxDay;
    
    if ($terminal eq 28) { $offset = 15; }
    elsif ($terminal eq 29) { $offset = 15; }
    else { $offset = 14; }
  } else { 
    $im->string(gdSmallFont,390,6,"$date",$purple); 
    $terminal = 7;
  }
  $im->string(gdSmallFont,462,17,"$nas",$gray);

  # draw horizontal numbers and lines
  for (local $i=0; $i < $terminal; $i++) {
    if ($mode =~ /monthly/) {
      local $ii = $i + 1;
      if (($i % 2)==0) {
	if ($ii > 10) {
	  $im->string(gdSmallFont,35+$offset*$i,282,"$ii",$blue);
	} else {
	  $im->string(gdSmallFont,38+$offset*$i,282,"$ii",$blue);
	}
      }
      $im->line(40+$offset*$i,276,40+$offset*$i,281,$black);
    } else {
      $im->string(gdSmallFont,72+55*$i,282,"$daysList{$i}",$blue);
      $im->line(80+55*$i,276,80+55*$i,281,$black);
    }
  }

  # draw vertical numbers and lines
  local $pos = 271-15;
  $yMax += $yMax;

  local $diff = (int((int($yMax/5)+1)*5/16/5)+1)*5;
  local $cnt = $diff;
  for (local $i=0; $i < 16; $i++) {
    if ($cnt < 10) { $im->string(gdSmallFont,20,$pos,"$cnt",$blue); }
    elsif (($cnt >= 100) && ($cnt < 1000)) 
      { $im->string(gdSmallFont,8,$pos,"$cnt",$blue); }
    elsif ($cnt > 1000) { $im->string(gdSmallFont,2,$pos,"$cnt",$blue); }
    else { $im->string(gdSmallFont,14,$pos,"$cnt",$blue); }
    $im->line(28,$pos+7,32,$pos+7,$black);
    $pos -= 15;
    $cnt += $diff;
  }

  $cnt -= $diff;
  local @arr = split /:/,"$data:";
  local $gfx_horiz;
  local $gfx_vert_new;
  local $gfx_vert_last1 = 500;
  local $gfx_vert_last2 = 500;
  local $gfx_vert_last3 = 500;
  for ($i=0; $i < $terminal; $i++) {
    local @int = split /,/, $arr[$i];
    local $num1 = $int[0];
    local $num2 = $int[1];
    local $num3 = $num1 + $num2;

    # draw it
    if (!($mode =~ /monthly/)) { $gfx_horiz = 40+$offset*$i; }
    else { $gfx_horiz = 40+$offset*$i-$offset; }

    # analog
    $gfx_vert_new = 278 - int($num1/($cnt/240));
    if ($gfx_vert_last1!=500) {
      $im->line($gfx_horiz, $gfx_vert_last1, 
		$gfx_horiz+$offset, $gfx_vert_new, $red);
    }
    $gfx_vert_last1 = $gfx_vert_new;

    # isdn
    $gfx_vert_new = 278 - int($num2/($cnt/240));
    if ($gfx_vert_last2!=500) {
      $im->line($gfx_horiz, $gfx_vert_last2, 
		$gfx_horiz+$offset, $gfx_vert_new, $green);
    }
    $gfx_vert_last2 = $gfx_vert_new;

    # total
    $gfx_vert_new = 278 - int($num3/($cnt/240));
    if ($gfx_vert_last3!=500) {
      $im->line($gfx_horiz, $gfx_vert_last3, 
		$gfx_horiz+$offset, $gfx_vert_new, $blue);
    }
    $gfx_vert_last3 = $gfx_vert_new;
  }

  open (OUTPUTFILE, ">$gfxDst") or err(12, "$gfxDst");
  binmode STDOUT;
  print OUTPUTFILE $im->gif;
  close OUTPUTFILE;

  dd "Created usage graph.";
}

#-----------------------------------------------
# Create Weekly Report Graphics *modem usage*
#===============================================
sub createWeek_ModemGfx {
  local ($gfxDst, $yMax, $data, $nas, $date, $mode) = @_;

  if ($mode =~ /monthly/) {
    draw_init_modem_usage($nas,"", "monthly", $date);
  } else { draw_init_modem_usage($nas,"", "week", $date); }
  local @arr = split /:/,"$data:";
  local $j = 0;

  for (local $i=1; $i <= $TOTAL_ANALOG; $i++) {
    local $usage = $arr[$j];
    $j++;

    if ($i < 10) { $graph_left = 17+18*$i }
    else { $graph_left = 20+18*$i }
    $graph_top = int ( 279 - $usage*2.4);

    $im->filledRectangle($graph_left, $graph_top, 
			 $graph_left+10, 276, gdTiled); 
    $im->rectangle($graph_left, $graph_top-1,
		   $graph_left+11, 278, $black); 
  }

  open (OUTPUTFILE, ">$gfxDst") or err(12, "$gfxDst");
  binmode STDOUT;
  print OUTPUTFILE $im->gif;
  close OUTPUTFILE;

  dd "Created modem usage graph.";
}

#-----------------------------------------------
# Create Monthly Report
#===============================================
sub createMonthlyReport {
  local ($startMonth) = @_;

  local ($year, $mon) = split /-/, $startMonth;
  dd "Inspecting month of $year-$mon...";

  createWeeklyReport("", "month", $year, $mon);
}

#-----------------------------------------------
# Create 2-year Report
#===============================================
sub create2yearReport {
  
  dd "Going to create report from $YEAR_REPORT_START.";
  local $regenerate = 0;
  if (!(-r "${DATADIR}year.data")) { $regenerate = 1; } 
  else {
    open (REPORT, "${DATADIR}year.data") or err(11, "${DATADIR}year.data");
    $line = <REPORT>;
    if (!($line =~ /$YEAR_REPORT_START/)) { $regenerate = 1; }
    while ($line ne "") {
      $lastline = $line;
      $line = <REPORT> || last;
    }

    $lastline =~ /(\d+)\ (-|\d+)/;
    if (int($1) < ($YEAR_REPORT_START*10)) {
      $lastline = "${YEAR_REPORT_START}0000";
    }
    close REPORT;
  }

  local $time = scalar time;
  $time -= 3600*24;
  @time = localtime($time);
  $time[5] += 1900;
  $time[4]++;
  local $todayYear = "$time[5]";
  local $todayMon  = "$time[4]";
  local $todayDay  = "$time[3]";
  
  local $endYear = $YEAR_REPORT_START + 2;  # 2 == 2 years
  local $curYear = $YEAR_REPORT_START;
  local $curMon  = 1;
  local $curDay  = 0;
  
  local $febDays1 = $YEAR_REPORT_START % 4;
  if ($febDays1==0) { $febDays1 = 29; } # skudd-aar
  else { $febDays1 = 28; }
  local $febDays2 = ($YEAR_REPORT_START+1) % 4;
  if ($febDays2==0) { $febDays2 = 29; } # skudd-aar
  else { $febDays2 = 28; }
  
  if ($regenerate==1) {
    dd "Regenerating file.";
    open (OUT, ">${DATADIR}year.data") or err(12,"${DATADIR}year.data");
    printf OUT "$YEAR_REPORT_START\n";
  } else { 
    $lastline =~ /(\d\d\d\d)(\d\d)(\d\d)/;

    if ((int($1)==$todayYear) && (int($2)==$todayMon)
	&& (int($3)==$todayDay)) { 
      $regenerate = 1;
      open (OUT, ">${DATADIR}year.data") or err(12,"${DATADIR}year.data");
      printf OUT "$YEAR_REPORT_START\n";
      dd "Run previously today. Regenerating.";
    } else {
        
      $lastline =~ /(\d\d\d\d)(\d\d)(\d\d)/;

      $curYear = int($1);
      $curMon = int($2);
      $curDay = int($3);
      local $rmon = $curMon;
      if ($curMon < 10) { $rmon = "0$rmon"; }
      $maxDay = $ndays{$rmon};

      if ($maxDay==$curDay) {
	$curMon++;
	if ($curMon>12) {
	  $curYear++;
	  $curMon = 1;
	}
	$curDay = 1;
      } else { $curDay++; }
      if ($curDay==1) { $curDay = 0; }
      open (OUT, ">>${DATADIR}year.data") or err(12,"${DATADIR}year.data");
      dd "Using fast method to generate."; 
    }
  }
  
  local $rmon = $curMon;
  if ($curMon < 10) { $rmon = "0$rmon"; }
  $maxDay = $ndays{$rmon};

  while ($YEAR_REPORT_START < $endYear) {
    
    if ($curDay==1) {
      local $rmon = $curMon;
      if ($curMon < 10) { $rmon = "0$rmon"; }
      $maxDay = $ndays{$rmon};
      $curDay++;
    } elsif ($curDay>$maxDay) {
      $curDay = 1;
      $curMon++;
      if ($curMon > 12) {
	$curMon = 1;
	$curYear++;
      }
    } elsif ($curDay==0) { $curDay = 1; }
    
    local $rmon = $curMon;
    if ($curMon < 10) { $rmon = "0$rmon"; }
    local $rday = $curDay;
    if ($curDay < 10) { $rday = "0$rday"; }
    $hours = "-";
    if (-r "${DATADIR}$curYear-$rmon-$rday.data") {
      open (LOG, "${DATADIR}$curYear-$rmon-$rday.data");
      $line = <LOG>;
      $hours = 0;
      while ($line ne "") {
	if ($line =~ /-no-hours/) {
	  $line =~ /<\d+-no-hours>(\d+)<\/>/;
	  $hours += $1;
	}
	$line = <LOG> || last;
      }
      close LOG;
    }
    printf OUT "$curYear$rmon$rday $hours\n";
    
    if (($curYear==$todayYear) && ($curMon==$todayMon)
	&& ($curDay==$todayDay)) { last; }
    if ($curDay!=1) { $curDay++; }
  }
  close OUT;
  
  # now, let's generate the graphics here :)

  open (YEARFILE, "${DATADIR}year.data") or err(11, "${DATADIR}year.data");
  $line = <YEARFILE>;
  $yMax = 0;
  while ($line ne "") {
    if ($line =~ /\d+\ (\d+|-)/) {
      if (($1 ne "-") && (int($1) > $yMax)) { $yMax = $1; }
    }
    $line = <YEARFILE> || last;
  }
  seek YEARFILE, 0, 0;    # rewind file
  $line = <YEARFILE>;
  local $year1 = int($line);
  local $year2 = $year1 + 2;

  open (INIT_YEAR, $GRAPH_INITIAL_YEAR) or
    err(11, "$GRAPH_INITIAL_YEAR");
  $im = newFromGif GD::Image (INIT_YEAR);
  close INIT_YEAR;
  
  ($white,$black,$red,$blue,$purple,$gray,$green) = 
    (
     $im->colorAllocate(255, 255, 255),
     $im->colorAllocate(0, 0, 0),
     $im->colorAllocate(255, 0, 0),
     $im->colorAllocate(0,0,255),
     $im->colorAllocate(250,50,250),
     $im->colorAllocate(180,180,180),
     $im->colorAllocate(0,180,0)
    );
  $im->transparent($white);       # white color is transparent
  
  # print message to where statistics are taken from
  $im->string(gdTinyFont,570,20,$GRAPH_SIGNATURE,$black);

  local $offset = 55;
  $im->string(gdLargeFont,570,3,"Period: $year1 -> $year2",$purple); 
  
  # draw vertical numbers and lines
  local $pos = 271-15;

  local $diff = (int((int($yMax/5)+1)*5/16/5)+1)*5;
  local $cnt = $diff;
  for (local $i=0; $i < 16; $i++) {
    if ($cnt < 10) { $im->string(gdSmallFont,20,$pos,"$cnt",$blue); }
    elsif (($cnt >= 100) && ($cnt < 1000)) 
      { $im->string(gdSmallFont,8,$pos,"$cnt",$blue); }
    elsif ($cnt > 1000) { $im->string(gdSmallFont,2,$pos,"$cnt",$blue); }
    else { $im->string(gdSmallFont,14,$pos,"$cnt",$blue); }
    $im->line(28,$pos+7,32,$pos+7,$black);
    $pos -= 15;
    $cnt += $diff;
  }
  $cnt -= $diff;

  $line = <YEARFILE>;
  $hpos = 50;
  $circular = 0;
  $lastPOSblack = 278;
  $lastPOSred = 278;
  while ($line ne "") {
    $line =~ /(\d\d\d\d)(\d\d)(\d\d)\ (\d+|-)/;
    local $mon = int($2);
    local $day = int($3);
    local $val = $4;
    
    if ($day==1) {
      $im->line($hpos,38,$hpos,275,$gray);
      $im->line($hpos,276,$hpos,281,$black);
      if ($mon < 10) { $realMon = "0$mon"; }
      else { $realMon = "$mon"; }
      $im->string(gdSmallFont,$hpos,282,"$RmonList{$realMon}",$blue);
    } 

    if ($val ne "-") {
      $val = int($val);
      $gfx_vert_new = 278 - int($val/($cnt/240));
      $im->line($hpos, $lastPOSblack, $hpos, 
		$gfx_vert_new, $black);
      $lastPOSblack = $gfx_vert_new;
    } else { $val = 0; }

    $values[$circular] = int($val);
    $circular = ($circular+1) % 4;
    if (($values[0]!=0) && ($values[1]!=0) &&
	($values[2]!=0) && ($values[3]!=0)) {
      local $tot = ($values[0]+$values[1]+$values[2]+$values[3])/4;
      $gfx_vert_new = 278 - int($tot/($cnt/240));
      $im->line($hpos, $lastPOSred, $hpos, 
		$gfx_vert_new, $red);
      $lastPOSred = $gfx_vert_new;
    }

    $hpos++;
    $line = <YEARFILE> || last;
  }

  open (OUTPUTFILE, ">${GFXDIR}2year.gif") or err(12, "${GFXDIR}2year.gif");
  binmode STDOUT;
  print OUTPUTFILE $im->gif;
  close OUTPUTFILE;
  
  dd "Created 2-year graph.";
}

#-----------------------------------------------
# Update the HTML indexes
#===============================================
sub updateHTMLidx {

  local @arr;
  local $arrIdx = 0;

  local @Dirarr;
  local $DirIdx = 0;
  local $curIDD = 0;

  opendir( DIRHAN, "${HTMLDIR}" );
  $line = readdir(DIRHAN);
  while ($line ne "") {

      if (-d "${HTMLDIR}$line") { 
	  if ($line =~ /(\d\d\d\d)/) {
	      $Dirarr[$DirIdx] = $line;
	      $DirIdx++;
	  }
      }

    if (!( ($line =~ /index\.html/) || 
	  ($line =~ /recently\.html/))) {
      if ($line =~ /month_(\d+)-(\d+).html/) {
	for ($walk = 0; $walk != $arrIdx; $walk++) {
	  if ($arr[$walk]{year}==$1) { last; }
	}
	if ($walk == $arrIdx) {
	  $arr[$arrIdx]{year} = $1;
	  $arr[$arrIdx]{mon} .= "$2,";
	  $arrIdx++;
	} else { $arr[$walk]{mon} .= "$2,"; }
	
      } elsif ($line =~ /week_(\d+)-(\d+)-(\d+)_(\d+)-(\d+)-(\d+).html/) {
	for ($walk = 0; $walk != $arrIdx; $walk++) {
	  if ($arr[$walk]{year}==$1) { last; }
	}
	if ($walk == $arrIdx) {
	  $arr[$arrIdx]{year} = $1;
	  $arr[$arrIdx]{week} .= "$2-$3-$4-$5-$6,";
	  $arrIdx++;
	} else { $arr[$walk]{week} .= "$2-$3-$4-$5-$6,"; }
	
      } elsif ($line =~ /(\d+)-(\d+)-(\d+).html/) {
	for ($walk = 0; $walk != $arrIdx; $walk++) {
	  if ($arr[$walk]{year}==$1) { last; }
	}
	if ($walk == $arrIdx) {
	  $arr[$arrIdx]{year} = $1;
	  $arr[$arrIdx]{day} .= "$2-$3,";
	  $arrIdx++;
	} else { $arr[$walk]{day} .= "$2-$3,"; }
	
      }
    }

      $line = readdir(DIRHAN);
      if (!defined($line)) { $line = ""; }
      if (($line eq "")) {
	  if ($curIDD < $DirIdx) {
	      closedir(DIRHAN);
	      opendir( DIRHAN, "${HTMLDIR}/$Dirarr[$curIDD]/" );
	      $curIDD++;
	      $line = readdir(DIRHAN) ;
	  }
     }      
  }
  
  closedir(DIRHAN);
  
  local $i=0;
  while ($i < $arrIdx) {

    open_html_file("${HTMLDIR}by_day_$arr[$i]{year}.html", 
		   "${HTML_INDEX_DAY_HEAD}");
    htmlfile_put("<h2>Year $arr[$i]{year}</h2>");
    if (defined $arr[$i]{day}) {
      local @splatt = sort split /,/,$arr[$i]{day};
      local $curSplatt = shift @splatt;
      htmlfile_put("<ul><ul>");
      $curMonth = "20";   # non-existing
      while ($curSplatt ne "") {
	local @tempDay = split /-/,$curSplatt;
	if ($curMonth ne $tempDay[0]) {
	  $curMonth = $tempDay[0];
	  htmlfile_put("</ul>$RmonListEx{$curMonth}<ul>");
	}
	local $tempVal = int( $tempDay[1] );
	if ($tempVal==1) { $tempVal = "1st"; }
	elsif ($tempVal==2) { $tempVal = "2nd"; }
	elsif ($tempVal==3) { $tempVal = "3rd"; }
	else { $tempVal = "${tempVal}th"; }
	
	htmlfile_put("<dd><a href=\"$arr[$i]{year}/$arr[$i]{year}-$curSplatt.html\">$tempVal</a>");
	$curSplatt = shift @splatt || last;
      }
      htmlfile_put("</ul></ul>");
    }
    htmlfile_put("<hr size=1>");    
    close_html_file("$HTML_INITIAL_TAIL");

    open_html_file("${HTMLDIR}by_week_$arr[$i]{year}.html", 
		   "${HTML_INDEX_WEEK_HEAD}");
    htmlfile_put("<h2>Year $arr[$i]{year}</h2>");

    if (defined $arr[$i]{week}) {
      local @splatt = sort split /,/,$arr[$i]{week};
      local $curSplatt = shift @splatt;
      htmlfile_put("<ul><ul>");

      $curMonth = "20";   # non-existing
      while ($curSplatt ne "") {

	local @tempDay = split /-/,$curSplatt;
	if ($curMonth ne $tempDay[0]) {
	  $curMonth = $tempDay[0];
	  htmlfile_put("</ul>$RmonListEx{$curMonth}<ul>");
	}

	local $tempVal = int( $tempDay[1] );
	if ($tempVal==1) { $tempVal = "1st"; }
	elsif ($tempVal==2) { $tempVal = "2nd"; }
	elsif ($tempVal==3) { $tempVal = "3rd"; }
	else { $tempVal = "${tempVal}th"; }
	local $tempVal2 = int( $tempDay[4] );
	if ($tempVal2==1) { $tempVal2 = "1st"; }
	elsif ($tempVal2==2) { $tempVal2 = "2nd"; }
	elsif ($tempVal2==3) { $tempVal2 = "3rd"; }
	else { $tempVal2 = "${tempVal2}th"; }
	
	htmlfile_put("<dd><a href=\"$arr[$i]{year}/week_$arr[$i]{year}-$tempDay[0]-$tempDay[1]_$tempDay[2]-$tempDay[3]-$tempDay[4].html\">${tempVal} to ${tempVal2} of $RmonListEx{$tempDay[3]}</a>");
	$curSplatt = shift @splatt || last;
      }
      htmlfile_put("</ul></ul>");
    }
    htmlfile_put("<hr size=1>");    
    close_html_file("$HTML_INITIAL_TAIL");

    open_html_file("${HTMLDIR}by_month_$arr[$i]{year}.html", 
		   "${HTML_INDEX_MONTH_HEAD}");
    htmlfile_put("<h2>Year $arr[$i]{year}</h2>");
    if (defined $arr[$i]{mon}) {

      local @splatt = sort split /,/,$arr[$i]{mon};
      local $curSplatt = shift @splatt;
      htmlfile_put("<ul>");
      while ($curSplatt ne "") {
	
	htmlfile_put("<dd><a href=\"$arr[$i]{year}/month_$arr[$i]{year}-$curSplatt.html\">$RmonListEx{$curSplatt}</a>");
	$curSplatt = shift @splatt || last;
      }
      htmlfile_put("</ul>");
    }
    htmlfile_put("<hr size=1>");    
    close_html_file("$HTML_INITIAL_TAIL");
    
    $i++;
  }

  open_html_file("${HTMLDIR}day.html", 
		   "${HTML_INDEX_DAY_HEAD}");
  htmlfile_put("<h2>Please select year</h2>");
  htmlfile_put("<ul>");
  for ($walk = 0; $walk != $arrIdx; $walk++) {
    htmlfile_put("<dd><h3><a href=\"by_day_$arr[$walk]{year}.html\">$arr[$walk]{year}</a></h3>");
  }
  htmlfile_put("</ul>\n<hr size=1>");
  close_html_file("$HTML_INITIAL_TAIL");
  

  open_html_file("${HTMLDIR}month.html", 
		   "${HTML_INDEX_MONTH_HEAD}");
  htmlfile_put("<h2>Please select year</h2>");
  htmlfile_put("<ul>");
  for ($walk = 0; $walk != $arrIdx; $walk++) {
    htmlfile_put("<dd><h3><a href=\"by_month_$arr[$walk]{year}.html\">$arr[$walk]{year}</a></h3>");
  }
  htmlfile_put("</ul>\n<hr size=1>");
  close_html_file("$HTML_INITIAL_TAIL");
  
  open_html_file("${HTMLDIR}week.html", 
		   "${HTML_INDEX_WEEK_HEAD}");
  htmlfile_put("<h2>Please select year</h2>");
  htmlfile_put("<ul>");
  for ($walk = 0; $walk != $arrIdx; $walk++) {
    htmlfile_put("<dd><h3><a href=\"by_week_$arr[$walk]{year}.html\">$arr[$walk]{year}</a></h3>");
  }
  htmlfile_put("</ul>\n<hr size=1>");
  close_html_file("$HTML_INITIAL_TAIL");

  if (!(-r "${HTMLDIR}day.html")) {
    system ("/bin/cp init/day.html ${HTMLDIR}");
  }
  if (!(-r "${HTMLDIR}index.html")) {
    system ("/bin/cp init/index.html ${HTMLDIR}");
  }
  if (!(-r "${HTMLDIR}month.html")) {
    system ("/bin/cp init/month.html ${HTMLDIR}");
  }
  if (!(-r "${HTMLDIR}week.html")) {
    system ("/bin/cp init/week.html ${HTMLDIR}");
  }

  dd "Updated all HTML index files.";
}

#-----------------------------------------------
# Init check
#===============================================
sub init_check
{
  if (!(-e "${DATADIR}")) {
    dd "Creating data dir. Probably deleted.";
    mkdir "${DATADIR}", -1;
  }
  if (!(-e "${HTMLDIR}")) {
    dd "Creating html dir. Probably deleted.";
    mkdir "${HTMLDIR}", -1;
  }
  if (!(-e "${GFXDIR}")) {
    dd "Creating gfx dir. Probably deleted.";
    mkdir "${GFXDIR}", -1;
  }
  if (!(-e "${TMPDIR}")) {
    dd "Creating tmp dir. Probably deleted.";
    mkdir "${TMPDIR}", -1;
  }
}

#-----------------------------------------------
# Main
#===============================================
parse_config_file;
parse_args(@ARGV);
if ($LOGGINGENABLED!=1) {
    print "\n----------------------------;-----------------------------------\n";
    print "Dialup Statistics $VERSION | (c) 1998 University of Tromsoe   !\n";
    print "----------------------------;-----------------------------------\n";
}
init_check;
if ($function =~ /import/) {

  dd "Importing logs in prefix=$IMPORT_PREFIX\n\t\t$IMPORT_FOLDER\n";

  $IMPORT_FOLDER =~ /.*(.)$/;
  if ($1 =~ /\//) { chop $IMPORT_FOLDER; }

  opendir( DIRHAN, "${IMPORT_FOLDER}" );
  $line = readdir(DIRHAN);
  while ($line ne "") {
    while (-d "${IMPORT_FOLDER}/$line") 
      { $line = readdir(DIRHAN); }

    if ($line =~ /$IMPORT_PREFIX.*/) {

      undef @dat; 
 
      $LOGFILE1 = "${IMPORT_FOLDER}/$line";
      dd "Importing logfile \"$line\"...\n";
      $continuous = 0;
      parse;
      close_data_file;

      ${rdate} =~ /(\d\d\d\d).*/;
      $tmpyr = $1;
      if (!(-e "${HTMLDIR}$tmpyr")) {
	dd "Creating html dir $tmpyr.";
	mkdir "${HTMLDIR}$tmpyr", -1;
      }

      open_html_file("${HTMLDIR}$tmpyr/$rdate.html", "$HTML_INITIAL_HEADING");
      write_html;
      close_html_file("$HTML_INITIAL_TAIL");
    }
    $line = readdir(DIRHAN) || last;
  }
  closedir(DIRHAN);
  dd "Finished.\n";

} elsif ($function =~ /daily/) {            # create daily report
  dd "Creating daily report.";
  $continuous = 0;
  parse;
  close_data_file;

  ${rdate} =~ /(\d\d\d\d).*/;
  $tmpyr = $1;
  if (!(-e "${HTMLDIR}$tmpyr")) {
    dd "Creating html dir $tmpyr.";
    mkdir "${HTMLDIR}$tmpyr", -1;
  }

  open_html_file("${HTMLDIR}$tmpyr/$rdate.html", "$HTML_INITIAL_HEADING");
  write_html;
  close_html_file("$HTML_INITIAL_TAIL");
} elsif ($function =~ /weekly/) {      # create weekly report
  dd "Creating weekly report.";
  if (defined $overrideStartReport) {
    createWeeklyReport($overrideStartReport,"norm", 0, 0);
  } else {
    local $time = scalar time;
    $time -= 3600*24*7;
    @time = localtime($time);
    $time[5] += 1900;
    if ($time[4] < 10) { $time[4] = "0$time[4]"; }
    local $weekStart = "$time[5]-$time[4]-$time[3]";
    createWeeklyReport($weekStart, "norm", 0, 0);
  }
} elsif ($function =~ /monthly/) {     # create monthly report
  dd "Creating monthly report.";

  if (defined $overrideStartReport) {
    createMonthlyReport($overrideStartReport);
  } else {
    @time = localtime;
    $time[5] += 1900;
    if ($time[4]==1) {
      $time[4] = 12;
      $time[5]--;
    }
    if ($time[4] < 10) { $time[4] = "0$time[4]"; }
    local $monStart = "$time[5]-$time[4]-$time[3]";
    createMonthlyReport($monStart);
  }
} elsif ($function =~ /2year/) {       # create 2-year report
  dd "Creating 2-year report.";
  create2yearReport();  
} elsif ($function =~ /continuous/) {  # create continuous report
  dd "Creating continuous report.";
  if ($LOGFILE_SET != 1) { $LOGFILE1 = $LOGFILECONT; }
  $continuous = 1;
  parse;
  close_data_file;

  if (!(-e "${HTMLDIR}misc")) {
    dd "Creating html dir Misc ";
    mkdir "${HTMLDIR}misc", -1;
  }

  open_html_file("${HTMLDIR}misc/recently.html", "$HTML_INITIAL_HEADING_CONT");
write_html;
  close_html_file("$HTML_INITIAL_TAIL_CONT");
}
updateHTMLidx;
